####################################################################################
# -*- coding: utf-8 -*-                                                           #
#!/usr/bin/env python                                                             #
# SMETime Data Manager for SME's                                                  #
# Copyright (C) 2008 Anil anilet@gmail.com                                        #
#                                                                                 #
# This program is free software; you can redistribute it and/or                   #
# modify it under the terms of the GNU General Public License                     #
# version 2 as published by the Free Software Foundation.                         #
#                                                                                 #
# This program is distributed in the hope that it will be useful,                 #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                  #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                   #
# GNU General Public License for more details.                                    #
#                                                                                 #
# You should have received a copy of the GNU General Public License               #
# along with this program; if not, write to the Free Software                     #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. #
###################################################################################
import logging
#import sys
import os
import datetime
from math import *
import string
#from string import *
import base64
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets  import *
from PyQt5.QtXml import *
from PyQt5.QtSql import *
from images.images_resources_rc import *
from ui.ui_main import Ui_MainWindow
from matplotlib.ticker import MultipleLocator, FormatStrFormatter, NullFormatter
from ui.calendarwidgetinternal import CalendarWidgetInternal
from login import LoginDialog # 
from ui.setup import SetupDialog
from PyQt5.QtWebKit import *
from PyQt5 import QtPrintSupport
from PyQt5.QtNetwork import *
import erppeek


FORMAT = '[%(levelname)-7s] [%(name)-35s] - %(message)s'
logging.basicConfig(level=logging.DEBUG, format=FORMAT)
logger = logging.getLogger('SMETime')
appName = "SMETime Odoo Client"
appVersion = "2.1.0"

ID, FNAME, LNAME, POSITION = range(4)
WORKID = 0
JOBNO = 1
TASK = 2
WSUMMARY = 3
DATE = 7
WDAY = 4
WMONTH = 5
WYEAR = 6
EMPLID = 8
HOUR = 4
OVERTIME = 5
UHRS = 6
NOTES = 11
ACQUIRED = 1
WIDH = 20
WIDH1 = 80
ERPWORKID=13
EMPLOYEEID = ""
JOB_NO = ""
# Global variables

prevJobNo = ""
DATE_FORMAT = "MMM d, yyyy"
DATE_FORMAT1="yyyy-MM-d"

""" Timer specific constants """
TIMER_PERIOD        = 1000      # 1000 miliseconds - 1 second
TIMER_HOUR_PERIOD   = 60 * 60   # 3600 seconds = 1 hour
TIMER_MINUTE_PERIOD = 60        # 60 seconds = 1 minute

class SMETimeClient(QMainWindow, Ui_MainWindow):
    def __init__(self, parent=None ):
        QMainWindow.__init__(self, parent)
        self.setupUi(self)
        self.debug = True
        self.initialized = False
        self.sqlDbHost = ""
        self.sqlDbName = ""
        self.sqlDbUser = ""
        self.decodedPassword = ""
        self.erppeekUser = ""
        self.erppeekUserPassword = ""
        self.erppeekString = ""
        self.currentDate = None
        self.currentUserID = None
        self.currentJobNo = None
        self.currentUserName = None    # QString
        self.currentUserDisplayName = None
        self.currentMonth = None
        self.currentWeek = None
        self.newRow = True
        self.monthChanged = False
        self.dailyTimeDirty = False
        self.inTimeText = None
        self.outTimeText = None
        self.dataEntered = False
        self.dataSaved = False
        self.dailyTimeCalculated = False
        self.timeCardInDatabase = False
        self.callingFromProgram = False
        self.jobDataSaved = False
        self.firstRun = True
        self.chartDrawn = False
        self.progressTimer = QTimer(self)
        self.messageTimer = QTimer(self)
        self.dirty = True
        self.taskNotInDatabase = False
        self.workInDatabase = False
        self.projectId =""
        self.fmComboboxes.setVisible(False)
        self.leTimeCardId.setVisible(False)
        self.frame_timecard_info.setVisible(False)
        self.webLoaded = False
        self.progressTimer.timeout.connect(self.closeProgress)
        self.messageTimer.timeout.connect(self.hideMessage)
        if (self.getSettings()):
            self.connection = self.createConnection()
            if self.connection:
               self.loginDialog = LoginDialog(self)
               self.loginDialog.loginButton.clicked.connect(self.doLogin)
               self.loginDialog.adminButton.clicked.connect(self.loadWebview)
            else:
               self.setupDialog = SetupDialog(self)
               #self.SetuoDialog.stackedWidget.setCurrentIndex(2)
               self.setupDialog.setWindowTitle( "SMETime Odoo Client- Setup database")
        else:
            self.setupDialog = SetupDialog(self)
            #self.SetuoDialog.stackedWidget.setCurrentIndex(2)
            self.setupDialog.setWindowTitle( "SMETime Odoo Client- Setup database")
        
        

    def doLogin(self):
        """Calls login function from login module """
        if self.debug:
            logger.debug('doLogin from main') 
        validUser , valid = self.loginDialog.doLogin()
        if valid:
            self.currentUserName = validUser
            self.currentUserID = self.loginDialog.cbId.currentText()  
            self.getCurrentUserDisplayName(self.loginDialog.cbPartnerId.currentText())
            self.initializeApplication(self.currentUserID )
            self.show()
            self.loginDialog.hide()

           
          
    def loadCalendarWidgetView(self):
      
        """
            Loads custom calendar view to show work details of all employees for the current month
            We can load different munth by selecting it from the combo box
        """
        
        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
        self.lblMessage.setText("Loading Monthly work details")
        self.progressMessage.setVisible(True)
        self.progressMessage.setValue(0)
        self.fillMonthCombo() #Loads months to combo box
        self.calendarWidget = CalendarWidgetInternal(self.emplWorkCalWidget,  self.currentUserID)
        self.calendarWidget.setGeometry(self.emplWorkCalWidget.geometry())
        now = datetime.datetime.now()
        day = now.day
        year = now.year
        id = self.currentUserID
        self.progressMessage.setValue(5)
        self.cbMonth.currentIndexChanged.connect(self.redrawEmployeeWorkCalendar)
        self.progressMessage.setValue(10)
        
        #self.connect(self.calendarWidget, SIGNAL('selectedDateChanged(QDate)'), self.setDateForWorkView)
        #self.calendarWidget.selectedDateChanged().connect(self.setDateForWorkView)
        self.progressMessage.setValue(40)
        month = self.cbMonth.currentIndex() +1
        year = self.cbYear.currentText()
        self.calendarWidget.setSelectedMonth(month, year)
        #self.calendarWidget.selectedDateChanged().connect(self.setDateForWorkView)
        
        self.progressMessage.setValue(60)        
        self.lblMessage.setText("Selcet a date from the calendar to view work details")
        self.loadHourViewForChart()
        self.progressMessage.setValue(70)
        self.hourModelChart.setFilter("worked_Day = %s and worked_Month = %s and worked_Year = %s and FK_empl_Id = %s" % (day, month, year, id))
        #self.calendarWidget.setSelectedMonth(month)
        self.stackedWidget.setCurrentIndex(1)
        self.progressMessage.setValue(80)
        self.buttonMonthlyWorkView.setDisabled(True)
        self.progressMessage.setValue(100)
        self.lblMessage.setText("Showing  Monthly work details  for  %s "%self.cbMonth.currentText())
        self.cbMonth.setVisible(True)
        self.progressTimer.start(1000)
        QApplication.restoreOverrideCursor()
        
        
    
    def loadHourViewForChart(self):
        """ Initializes table view to show work details 
            when user clicks on a cell in custom calendar view
        """
        self.hourModelChart = QSqlRelationalTableModel(self)
        self.hourModelChart.setTable("smeclient_work")
        self.hourModelChart.setHeaderData(1, Qt.Horizontal, QVariant("Job No"))
        self.hourModelChart.setHeaderData(2, Qt.Horizontal,   QVariant("Task Description"))
        self.hourModelChart.setHeaderData(3, Qt.Horizontal,  QVariant("Work Summary"))
        self.hourModelChart.setHeaderData(4, Qt.Horizontal,  QVariant("Worked Hours"))
        self.hourModelChart.setHeaderData(5, Qt.Horizontal,  QVariant("Overtime Hours"))
        self.hourModelChart.setHeaderData(6, Qt.Horizontal,  QVariant("Unattended Hours"))
        self.hourModelChart.setEditStrategy(QSqlTableModel.OnManualSubmit)
        self.hourModelChart.select()
        self.twEmployeeDailyWork.setModel(self.hourModelChart)
        self.twEmployeeDailyWork.setSelectionMode(QTableView.SingleSelection)
        self.twEmployeeDailyWork.setSelectionBehavior(QTableView.SelectRows)
        self.twEmployeeDailyWork.resizeColumnsToContents()
        self.twEmployeeDailyWork.setColumnHidden(0, True)
        self.twEmployeeDailyWork.setColumnHidden(7, True)
        self.twEmployeeDailyWork.setColumnHidden(8, True)
        self.twEmployeeDailyWork.setColumnHidden(9, True)
        self.twEmployeeDailyWork.setColumnHidden(10, True)
        self.twEmployeeDailyWork.setColumnHidden(11, True)
        self.twEmployeeDailyWork.setColumnHidden(12, True)
        self.twEmployeeDailyWork.setColumnHidden(13, True)
        self.twEmployeeDailyWork.setColumnHidden(14, True)
        self.twEmployeeDailyWork.setAlternatingRowColors(True)
        self.twEmployeeDailyWork.resizeColumnsToContents()
        self.twEmployeeDailyWork.setColumnWidth(1, 60)
        self.twEmployeeDailyWork.setColumnWidth(2, 280)
        self.twEmployeeDailyWork.setColumnWidth(3, 280)       
        self.twEmployeeDailyWork.horizontalHeader().setStretchLastSection(True)
        self.twEmployeeDailyWork.setEditTriggers(QAbstractItemView.NoEditTriggers)
    
    
    
    def redrawEmployeeWorkCalendar(self):
        """ 
           Redraws work calendar
           
        """
        if self.debug:
            logger.debug('Redrawing work calendar') 
        month = self.cbMonth.currentIndex() +1
        year = self.cbYear.currentText()
        self.calendarWidget.setSelectedMonth(month,year)
   
        
    def setDateForWorkView(self):
        #date = selectedDate
        
        day = self.calendarWidget.selectedDay()
        user = self.calendarWidget.selectedUser()
        id = self.getUserIDForChart(user)
       # self.currentMonth = date.month()
        #print "Setting date for view day is %s user is %s" %( day,  id)
        dt = self.cbMonth.currentText()
        startSplit = split(dt,"-")
        m = startSplit[0]
        year= startSplit[1]
        if m=="January":
            month = 1
        elif m =="February":
            month = 2
        elif m =="March":
            month = 3
        elif m =="April":
            month = 4
        elif m =="May":
            month = 5
        elif m =="June":
            month = 6
        elif m =="July":
            month = 7
        elif m =="August":
            month = 8
        elif m =="September":
            month = 9
        elif m =="October":
            month = 10
        elif m =="November":
            month = 11
        else: 
            month = 12
        
        #year = 2012
        self.lblDate.setText("Selected Date : %s-%s-%s" % (day, month, year))
      #  pydate = date.toPyDate()
        self.hourModelChart.setFilter("worked_Day = %s and worked_Month = %s and worked_Year = %s and FK_empl_Id = %s" % (day, month, year, id))
        self.lblMessage.setText("Browsing work details of %s on %s-%s-%s" %(user,  day,  month,  year))


        
    def fillMonthCombo(self):
        #print"Filling Month"
        now = datetime.datetime.now()
        month = now.strftime("%B")
        
        #print"Month is %s"%month
        self.cbYear.clear()
        year = now.year
        yesterYear = year - 1
        self.cbYear.addItem(str(year))
        self.cbYear.addItem(str(yesterYear))
        month = "%s" %(month)
        self.cbMonth.clear()
        #print "Year is %s"%year
        strMonth = ["January", "February",  "March",  "April",  "May",  "June",  "July",  "August",  "September", "October",  "November", "December"]
        for x in range (len(strMonth)):
            name = "%s" %(strMonth[x])
            self.cbMonth.addItem(name)
        self.cbMonth.setCurrentIndex(self.cbMonth.findText(month))

    def loadWebview(self):
        """
            Loads web view with oodo, asks for user name and password
            To investigate to pass user name to web view
        """
        if self.debug:
            logger.debug('loadWebview') 
        self.stackedWidget.setCurrentIndex(2)
        if not self.webLoaded:
            self.fmMenu.setVisible(False)
            self.fmMessage.setVisible(False)
            self.fmStatus.setVisible(False)
            #self.stackedWidget.setVisible(False)
            #web = QWebView(self.centralwidget)
            self.webView.settings().setAttribute(QWebSettings.JavascriptEnabled, True)
            self.webView.settings().setAttribute(QWebSettings.AutoLoadImages, True)
            self.webView.settings().setAttribute(QWebSettings.JavascriptCanOpenWindows, True)
            self.webView.settings().setAttribute(QWebSettings.JavascriptCanAccessClipboard, True)
            self.webView.settings().setAttribute(QWebSettings.PluginsEnabled, True)
            self.webView.settings().setAttribute(QWebSettings.OfflineWebApplicationCacheEnabled, True)
            appCacheDir = QtCore.QDir.homePath()
            self.webView.settings().setOfflineWebApplicationCachePath(appCacheDir)
            self.webView.settings().setOfflineWebApplicationCacheQuota(50 *1024 * 1024)
            self.webView.settings().setAttribute(QWebSettings.LocalStorageEnabled, True)
            self.webView.settings().setLocalStoragePath(appCacheDir)
            self.webView.settings().setAttribute(QWebSettings.LinksIncludedInFocusChain, True)
            self.webView.settings().setAttribute(QWebSettings.LocalContentCanAccessRemoteUrls, True)
            self.webView.settings().setAttribute(QWebSettings.LocalContentCanAccessFileUrls, True)
            #web.settings().setAttribute(QWebSettings.JavascriptEnabled, True)
            
            #self.verticalLayout_11.addWidget(web)
            #print self.SERVER
            server = "http://" + self.sqlDbHost + ":8069"
            #print("Server url is " )
            #print(server)
            self.webView.load(QUrl(server))
            message = " - Management Console"
            self.setWindowTitle(appName +" - " +  appVersion + message )
            self.webLoaded = True
            self.show()
            self.loginDialog.hide()
            #web.show()


  

#     ####pg
#    def fillTaskCombo(self):
#        if self.debug:
#            logger.debug('fillTaskCombo') 
#        jobNo = str(self.leJob_no.text())
#        self.taskNotInDatabase = False
#        jobExists = False
#        self.cbTask.clear()
#        self.cbTaskId.clear()
#        self.cbProjectId.clear()
#        self.taskNotInDatabase =False
#        print "Job No is: %s" %jobNo
#        if  len(jobNo) > 4  and len(jobNo) < 7:
#            jobExists = self.checkJobNoExists(jobNo)
#         #   print jobExists
#        if jobExists:
#            print "Inside querry jobExists"
#            query = QSqlQuery()
#            query.exec_("SELECT id, name, project_id FROM project_task WHERE   job_no = '%s'" % jobNo)
#            query.first()
#            if query.isValid():
#                query.previous()
#                print "Inside querry is valid"
#                while query.next():
#                    name = QString(query.value(1).toString())
#                    print name
#                    self.cbTask.addItem(QString(query.value(1).toString()))
#                    self.cbTaskId.addItem(QString(query.value(0).toString()))
#                    self.cbProjectId.addItem(QString(query.value(2).toString()))  
#                    self.cbTask.setVisible(True)
#                    self.leTask.setVisible(False)
#            else:
#                self.taskNotInDatabase = True
#                reply = QMessageBox.question(None, "No tasks defined for this job", "This job doe not have any task assigned yet, Please enter a brief name for the task to create one for you",
#                                             QMessageBox.StandardButtons(QMessageBox.Cancel | QMessageBox.No | QMessageBox.Yes))
#                if reply == QMessageBox.Yes:
#                    self.cbTask.setVisible(False)
#                    self.leTask.setVisible(True)
#                    self.leTask.setFocus()
#                    pass
#                   # self.createNewTaskInOpenerp(jobNo)
#            if not query.isActive():
#                QMessageBox.warning(self, "Daily time sheet -Database Error - fillTaskCombo",
#                    "Failed to open database, Please check your login details") 
#        else:
#            reply = QMessageBox.question(None, "Is this a new Job no?", "This job no does not exist in the database- Do you want to save it in the database?",
#                                             QMessageBox.StandardButtons(QMessageBox.Cancel | QMessageBox.No | QMessageBox.Yes))
#            if reply == QMessageBox.Yes:
#                self.projectId = self.createNewJobInOpenerp(jobNo)
#                if not self.projectId:
#                    QMessageBox.warning(self, "Daily time sheet-Database Error - Could not create new job in the database",
#                    "Could not save to database Please ask administrator to create the job for you")
#            else:
#                pass

#    def provideJobSearchDropDown(self):
#        if self.debug:
#            logger.debug('provideJobSearchDropDown')
#        query = QSqlQuery()
#        punctuationRe = QRegExp(r"^[Q0-9][0-9][0-9][0-9][0-9]?[ab]?")
#        queryString = "SELECT DISTINCT(job_no) FROM project_project WHERE job_no IS NOT NULL ORDER BY job_no DESC"
#        query.exec_(queryString)
#        jobWordList = QStringList()
#        while query.next():
#            jobWordList << QString(query.value(0).toString())
#        jobCompleter = QCompleter(jobWordList, self)
#        completionMode = QCompleter.PopupCompletion
#        jobCompleter.setCompletionMode(completionMode)
#        self.leJob_no.setValidator(QRegExpValidator(punctuationRe, self))
#        self.leJob_no.setCompleter(jobCompleter)


    def calculateRemainingHours(self):
        if self.debug:
            logger.debug('calculateRemainingHours ')
        if not len(self.leWorkedHours.text()) == 0:
            wh = float(self.leWorkedHours.text())
            #print wh
        else:
            wh = 0
        if not len(self.leDataEntered.text()) == 0:
            de = float(self.leDataEntered.text())
            #print de
        else:
            de = 0
        remainingHours = wh - de
        self.leRemainingHours.setText(str(remainingHours))


    def getCurrentUserDisplayName(self, partnerId):
        """
            Get current user display name and User picture  from Odoo
        """
        if self.debug:
            logger.debug('getCurrentUserDisplayName ')
            logger.debug("Current user partner id is %s"%partnerId)
        query = QSqlQuery()
        ba1 = QByteArray()
        pic = QPixmap()
        query.exec_("SELECT decode(replace(encode(image_small,'escape')::text,E'\012',''),'base64') as logo,  res_partner.name FROM res_partner WHERE res_partner.id IN (%s) AND (TRUE)"% ( partnerId))
        while query.next():
            self.currentUserDisplayName  = query.value(1)
            logger.debug("Current user partner id is %s"%self.currentUserDisplayName)
            ba1 = query.value(0)
        if ba1:
            if self.debug:
                logger.debug('Loading static image ')
            fileName = "user.png"

           # QImage image(fileName);
            #LBL_IMAGE->setPixmap(QPixmap::fromImage(image)); // Put image into QLabel object (optional)

            #// load image to bytearray
           # QByteArray ba;
            #file = QFile(fileName)
            #if(file.open(QIODevice.ReadOnly)):
                #ba1 = file.readAll()
            #file.close()

        pic.loadFromData( ba1)
        pic = pic.scaled(QSize(28,28),Qt.KeepAspectRatio,Qt.SmoothTransformation)
        self.lblUser.setPixmap(pic)
        self.btUser.setText(self.currentUserDisplayName )





    def getWorkedHours(self):
        """
            Get worked hours from database for the selected date
        """
        if self.debug:
            logger.debug('getWorkedHours ')
        date = self.currentDate
        pydate = date.toPyDate()
        dt_D = str(pydate.day)
        dt_M = str(pydate.month)
        dt_Y = str(pydate.year)
        dt = "%s-%s-%s"%(dt_Y, dt_M, dt_D)
        hours = "0"
        query = QSqlQuery()
        query.exec_("SELECT sum(worked_hours)+sum(overtime_hours) as hours FROM smeclient_work WHERE date = '%s' AND user_id = %s " % (dt, self.currentUserID))
        while query.next():
            hours  = query.value(0)            #return self.status
            #print ID
       # self._total_time
        self.leDataEntered.setText(str(hours))
        self.calculateRemainingHours()
       # return hours



#    def updateProjectTaskWork(self,  data,  id):
#        if self.debug:
#            logger.debug('updateProjectTaskWork ')
#        self.lblMessage.setText("Saving worked hours")
#        print 'Printing data and data is'
#        print data
#        date = self.currentDate
#        pydate = date.toPyDate()
#        dt_D = str(pydate.day)
#        dt_M = str(pydate.month)
#        dt_Y = str(pydate.year)
#        dt = "%s-%s-%s"%(dt_Y, dt_M, dt_D)
#        self.lblMessage.setText("Updating Worke hours")
#        query.prepare("UPDATE smeclient_work SET task = :task ,"
#        " work_summary = :ws, worked_hours = :wh, overtime_hours = :oth,"
#        " unattended_hours = :uh WHERE id = :tid")
#        query.bindValue(":task", QVariant(self.cbTask.currentText()))
#        query.bindValue(":ws", QVariant(self.leWorkSummery.text()))
#        query.bindValue(":wh", QVariant(self.leWorkSummery.text()))
#        query.bindValue(":oth", QVariant(self.leWorkSummery.text()))
#        query.bindValue(":uh", QVariant(self.leWorkSummery.text()))
#        query.bindValue(":tid", QVariant(self.cbTaskId.currentText()))
#        self.progressMessage.setValue(30)
#        query.exec_()
#        if not query.isActive():
#            QMessageBox.warning(self, "Daily time sheet -- Database Error",
#                    "Failed to open database %s" % (query.lastError().text()))
#            self.lblMessage.setText("Saving data failed")
#            self.progressMessage.setValue(0)
#        else:
#            self.progressMessage.setValue(60)
#            self.chartDrawn = False
#            self.btSaveTaskWork.setEnabled(False)
#            self.cbTimeSpent.setCurrentIndex(self.cbTimeSpent.findText("0.00")) 
#            self.cbOverTime.setCurrentIndex(self.cbOverTime.findText("0.00")) 
#            self.cbUnAttended.setCurrentIndex(self.cbUnAttended.findText("0.00")) 
#            self.progressTimer.start(1000)
#            self.progressMessage.setValue(100)
#            self.hourView.setEnabled(True)
#            self.lblMessage.setText("Work hours saved")


    def setOerpWorkIdInDatabase(self,  id,  oerpwid):
        if self.debug:
            logger.debug('setOerpWorkIdInDatabase ')
            logger.debug("Updating smeclient-work")
        query = QSqlQuery()
        query.prepare("UPDATE smeclient_work SET workId = :oerpwid  WHERE id = :id")
        query.bindValue(":oerpwid", QVariant(oerpwid))
        query.bindValue(":id", QVariant(id))
        query.exec_()
        if not query.isActive():
            self.lblMessage.setText("Saving data failed")
            return False
        else:
            self.lblMessage.setText("Work hours saved")
            return True



    def saveImageToDatabase(self,  data,  id):
        """
            Save image to database -- not used now
        """
        if self.debug:
            logger.debug('saveImageToDatabase ')
        query = QSqlQuery()
        query.prepare("UPDATE hr_timecard SET image = :img_data  WHERE id = :id")
        query.bindValue(":img_data", QVariant(data))
        query.bindValue(":id", QVariant(id))
        query.exec_()
        if not query.isActive():
            self.lblMessage.setText("Saving chart failed")
            return False
        else:
            self.lblMessage.setText("Work chart saved")
            return True

    #def enableSaveTaskWork(self):
        #if self.debug:
            #logger.debug('enableSaveTaskWork ')
        #jno = unicode(self.leJob_no.text())
        #ws = unicode(self.leWorkSummery.text())
        #if self.debug:
            #logger.debug('cbTimeSpent text is ')
            #logger.debug(self.cbTimeSpent.currentText())
            #logger.debug('cbOverTime text is ')
            #logger.debug(self.cbOverTime.currentText())
        #nt = 0
        #ot = 0
        #nt = float(self.cbTimeSpent.currentText())
        #ot = float(self.cbOverTime.currentText())
        #tothrs = nt + ot
       ## ut = unicode(self.cbUnAttended.currentText())
        #if  len(jno) == 0:
            #self.btSaveTaskWork.setEnabled(False)
            #self.leJob_no.setFocus()
            #return
        #elif  len(ws) == 0:
            #self.btSaveTaskWork.setEnabled(False)
            #self.leWorkSummery.setFocus()
           ## return
        #elif not tothrs == 0.00 :
            #self.btSaveTaskWork.setEnabled(True)
        #else:
            #self.cbTimeSpent.setFocus()
            #self.btSaveTaskWork.setEnabled(False)
            ##return


#
    def initializeApplication(self, userid):
        """
            Initialise application and all the variables, called from Login
        """
        if self.debug:
            logger.debug('loadNormalView ')
        self.currentDate = None
        self.currentUserID = userid
        self.currentMonth = None
        self.currentWeek = None
        self.oldTotalTime = None
        #self.currentDepartmentId = None
        self.monthChanged = False
        self.dailyTimeDirty = False
        self.inTimeText = None
        self.outTimeText = None
        self.dataEntered = False
        self.dataSaved = False
        self.dailyTimeCalculated = False
        self.timeCardInDatabase = True
        self.callingFromProgram = False
        self.callingFromProgramWork = False
        self.firstRun = True
        self.jobFilled = False
        self.chartDrawn = False
        self.cbMonth.setVisible(False)
        message = " - Time Entry"
        self.setWindowTitle(appName +" - " +  appVersion + message )
        #self.setWindowTitle("MainWindow", appName + appVersion)
        file = "settings.ini"
        settings = QSettings(file, QSettings.IniFormat)
        #self.restoreGeometry(settings.value("Geometry"))
        screen = QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width() - size.width()) / 2, (screen.height() - size.height()) / 2)
        self.showMaximized()
        icon = QIcon()
        icon.addPixmap(QPixmap(":/Icons/icons/lighttasks.png"), QIcon.Normal, QIcon.Off)
        self.setWindowIcon(icon)
        now = datetime.datetime.now()
        self.calenderDay.setSelectedDate(now)
        date = self.calenderDay.selectedDate()
        self.currentDate = self.calenderDay.selectedDate()
        self.calenderDay.setMaximumDate(now)
        punctuationRe = QRegExp(r"^[0-9]{0,2}[:.,]{1,1}[0-5]{0,1}[0-9]{0,1}$")
       # inWordList = QStringList()
        inWordList = ["7:30" , "16.00" , "00:00" , "00:30" , "1:00" , "1:30" , "2:00" , "2:30" , "3:00" , "3:30" , "4:00" , "4:30" , "5:00" , "5:30" , "6:00" , "6:30" , "7:00",  "8:00" , "8:30" , "9:00" , "9:30" , "10:00" , "10:30" , "11:00" , "11:30" , "12:00" , "12:30" ,   "13:00" ,  "13.30" ,  "14.00" , "14.30" , "15.00" , "15.30"  , "16.30" , "17.00" , "17.30" , "18.00" , "18.30" , "19:00" , "19.30" , "20.00" , "20.30" , "21.00" , "21.30" , "22.00" , "22.30" , "23.00" , "23.30" ]
        inCompleter = QCompleter(inWordList, self)
        completionMode = QCompleter.InlineCompletion
        inCompleter.setCompletionMode(completionMode)
        self.inLineEdit.setCompleter( inCompleter)
        outCompleter = QCompleter(inWordList, self)
        completionMode = QCompleter.InlineCompletion
        outCompleter.setCompletionMode(completionMode)
        self.outLineEdit.setCompleter( outCompleter)
        self.inLineEdit.setValidator( QRegExpValidator(punctuationRe, self))
        self.outLineEdit.setValidator( QRegExpValidator(punctuationRe, self))
        self.currentMonth = date.month()
        self.fmMessage.setVisible(False)
     #   self.leTask.setVisible(False)
        self.stackedWidget.setCurrentIndex(0)
        self.calenderDay.setSelectedDate(now)
        self.currentDate = self.calenderDay.selectedDate()
        self.hourModel = QSqlRelationalTableModel(self)
        self.hourModel.setTable("smeclient_work")
        self.hourModel.setHeaderData(1, Qt.Horizontal, QVariant("Job No"))
        self.hourModel.setHeaderData(2, Qt.Horizontal,   QVariant("Task Description"))
        self.hourModel.setHeaderData(3, Qt.Horizontal,  QVariant("Work Summary"))
        self.hourModel.setHeaderData(4, Qt.Horizontal,  QVariant("Worked Hours"))
        self.hourModel.setHeaderData(5, Qt.Horizontal,  QVariant("Overtime Hours"))
        self.hourModel.setHeaderData(6, Qt.Horizontal,  QVariant("Unattended Hours"))
        self.hourModel.select()
        self.hourView.setModel(self.hourModel)
        self.hourView.setSelectionMode(QTableView.SingleSelection)
        self.hourView.setSelectionBehavior(QTableView.SelectRows)
        self.hourView.resizeColumnsToContents()
        self.hourView.setColumnHidden(0, True)
        self.hourView.setColumnHidden(7, True)
        self.hourView.setColumnHidden(8, True)
        self.hourView.setColumnHidden(9, True)
        self.hourView.setColumnHidden(10, True)
        self.hourView.setColumnHidden(11, True)
        self.hourView.setColumnHidden(12, True)
        self.hourView.setColumnHidden(13, True)
        self.hourView.setColumnHidden(14, True)
        self.hourView.setAlternatingRowColors(True)
        self.hourView.resizeColumnsToContents()
        self.hourView.setColumnWidth(1, 60)
        self.hourView.setColumnWidth(2, 280)
        self.hourView.setColumnWidth(3, 280)
        self.hourModel.setEditStrategy(QSqlTableModel.OnManualSubmit)
        self.delegate = JobDelegate(self)
        #self.hourView.setItemDelegate(JobDelegate(self))
        self.hourView.setItemDelegate( self.delegate)
        self.lblDate.setText("   Selected Date : %s-%s-%s   " % (date.day(), date.month(), date.year()))
        self.lblMessage.setText("Select a date from the calender to enter Worked Hours")

        #setup signals
        #self.connect(self.calenderDay, SIGNAL('selectionChanged()'), self.dateChanged)
        self.calenderDay.selectionChanged.connect(self.dateChanged)
        #self.calenderDay.connect(self.calenderDay, SIGNAL('clicked(QDate)'), self._setMonthFilter)
        self.calenderDay.clicked.connect(self._setMonthFilter)
        self.inLineEdit.editingFinished.connect(self.saveDailyTime)
        self.outLineEdit.editingFinished.connect(self.saveDailyTime)
        self.noBreakCheckBox.stateChanged.connect(self.saveDailyTime)
        self.buttonNew.clicked.connect(self.addWork)
        self.btDeleteTaskWork.clicked.connect(self.deleteWork)
        self.hourView.clicked.connect(self.enableDeleteWork)
        #self.hourView.currentChanged.connect(self.test)
        self.hourView.clicked.connect(self.enableDeleteWork)
       # self.dayChartLabel.clicked.connect(self.on_pick)
        self.btSaveTaskWork.clicked.connect(self.submit)
        self.leWorkedHours.textChanged.connect(self.calculateRemainingHours)
        self.leDataEntered.textChanged.connect(self.calculateRemainingHours)
        self.hourView.model().dataChanged.connect(self.EnbleSave)
        self.btManage.clicked.connect(self.loadWebview)
        self.btTasks.clicked.connect(self.loadWebview)
        self.buttonMonthlyWorkView.clicked.connect(self.loadCalendarWidgetView)
        
        self.checkDailyTime()
        self._setMonthFilter(date)
        self.setDirty()
        self.setEnabled(True)
        




    def on_buttonEdit_clicked(self):
        self.btSaveTaskWork.setEnabled(True)


    def on_deleteTimeButton_clicked(self):
        self.deleteDailyTime()


    def on_editButton_clicked(self):
        self.inLineEdit.setEnabled(True)
        self.outLineEdit.setEnabled(True)
        self.noBreakCheckBox.setEnabled(True)

    def closeProgress(self):
        self.progressMessage.setVisible(False)

    def hideMessage(self):
        self.fmMessage.setVisible(False)




    #def createTask(self):
        #if self.debug:
            #logger.debug('createTask')
        #jobNo = str(self.leJob_no.text())
        #if  len(jobNo) > 4  and len(jobNo) < 7:
            #jobExists = self.checkJobNoExists(jobNo)
         ##   print jobExists
            #if jobExists:
                #self.createNewTaskInOpenerp(jobNo)
        #else:
            #self.leJob_no.setFocus()
            #return


    def setDirty(self):
        #print  "Printing self.inTimeText %s" %self.inTimeText
        #print  "Printing self.outTimeText %s" %self.outTimeText
        if not (self.inLineEdit.text() == self.inTimeText or self.outLineEdit.text() == self.outTimeText):
            self.dailyTimeDirty = True
            if self.debug:
                logger.debug("Text changed")
        else:
            self.dailyTimeDirty = False
            if self.debug:
                logger.debug("Text not changed")


    def createNewJobInOpenerp(self,  newJobNo):
        if self.debug:
            logger.debug('createNewJobInOpenerp-Creating new project for')
            logger.debug(newJobNo)
        try:
            self.progressMessage.setValue(00)
            self.progressMessage.setValue(10)
            project_data = {
                      'name': str(newJobNo),
                      'job_no': str(newJobNo)
                      }   
            self.progressMessage.setValue(50)
            self.insertDataToTable('project.project', project_data)
            self.progressMessage.setValue(100)
        except Exception as e:
            QMessageBox.warning(self, "Daily time sheet-Database Error",
                    "Error saving data to server- createNewJobInOpenerp, %s" %(e))


    def createNewTaskInOpenerp(self,  newJobNo,  taskName):
        if self.debug:
            logger.debug('createNewTaskInOpenerp-Creating new task for')
            #print newJobNo
        projectid = self.getProjectIdFromJobNo(newJobNo)  
        try:
            logger.debug('Trying oerp-new task no ')
            self.progressMessage.setValue(00)
            self.progressMessage.setValue(10)
            task_data = {
                        'name': str(taskName),
                        'project_id': str(projectid), 
                        'job_no': str(newJobNo)
                      }   
            self.progressMessage.setValue(50)
            newId = self.insertDataToTable('project.task', task_data)
            self.progressMessage.setValue(100)
            return newId
        except Exception as e:
            QMessageBox.warning(self, "Daily time sheet-Database Error- createNewTaskInOpenerp",
                  "Error saving data to server, %s" %(e))
                  
    
    def createDefaultTasksInOpenerp(self,  newJobNo):
        if self.debug:
            logger.debug('createDefaultTasksInOpenerp-Creating new task for')
        department = self.getDepartmentFromUserId()
        projectid = self.getProjectIdFromJobNo(newJobNo)  
        taskNameList = []
        checkJobNo = newJobNo[:1]
        if checkJobNo == "Q" or checkJobNo == "q":
            taskNameList.append("%s-Preliminary Design"%newJobNo)
            taskNameList.append("%s-Prepare Quotation"%newJobNo)
        elif department =="Manufacturing Manager":
            taskNameList.append("%s-Follow up"%newJobNo)
        elif department =="Tool maker" or department =="Senior Toolmaker" or department =="Apprentice Toolmaker":
            taskNameList.append("%s-Bench Work"%newJobNo)
            taskNameList.append("%s-Assembly"%newJobNo)
        elif department =="Designer":
            taskNameList.append("%s-Design"%newJobNo)
        elif department =="Wirecut Operator":
            taskNameList.append("%s-Wirecutting"%newJobNo)
        elif department =="NC Machinist":
            taskNameList.append("%s-NC Machining"%newJobNo)
        else:
            taskNameList.append("%s-NC Machining"%newJobNo)
            taskNameList.append("%s-Bench Work"%newJobNo)
            taskNameList.append("%s-Assembly"%newJobNo)
            taskNameList.append("%s-Design"%newJobNo)
        val = 40
        self.progressMessage.setValue(40)
        try:
            logger.debug('Trying oerp-new task no ')
            for x in taskNameList:
                task_data = {
                            'name': x,
                            'project_id': str(projectid), 
                            'job_no': str(newJobNo)
                          }   
                val = val + 10
                self.insertDataToTable('project.task', task_data)
                self.progressMessage.setValue(val)
        except Exception as e:
            QMessageBox.warning(self, "Daily time sheet-Database Error- createNewTaskInOpenerp",
                  "Error saving data to server, %s" %(e))




    def getProjectIdFromJobNo(self, newJobNo) :
        if self.debug:
            logger.debug('getProjectIdFromJobNo - getting project id for %s' %(newJobNo))
        query = QSqlQuery()
        ID = ""
        query.exec_("select id from project_project "
                "where job_no = '%s'" % newJobNo)
        while query.next():
            ID = str(query.value(0))
        if self.debug:
            logger.debug('getProjectId - project id is %s' %(ID))
        return ID

    def getTaskIdFromTaskName(self, name) :
        if self.debug:
            logger.debug('getTaskIdFromTaskName - getting project id for %s' %(name))
        query = QSqlQuery()
        ID = ""
        query.exec_("select id from project_task "
                "where name = '%s'" % name)
        while query.next():
            ID = str(query.value(0))
        if self.debug:
            logger.debug('getProjectId - project id is %s' %(ID))
        return ID

    def getWorkId(self) :
        if self.debug:
            logger.debug('getWorkId')
        query = QSqlQuery()
        ID = ""
        query.exec_("select max(id) from smeclient_work where user_id = %s" % self.currentUserID)
        while query.next():
            ID = str(query.value(0))
        if self.debug:
            logger.debug('getProjectId - project id is %s' %(ID))
        return ID
     
     
    def getEmployeeIdFromUserId(self) :
        if self.debug:
            logger.debug('getEmployeeIdFromUserId')
        query = QSqlQuery()
        ID = ""
        query.exec_("SELECT id FROM hr_employee WHERE name_related = '%s'" % self.currentUserDisplayName)
        if self.debug:
            logger.debug('Last query is %s' %(query.lastQuery()))
        while query.next():
            ID = str(query.value(0))
        if self.debug:
            logger.debug('getEmployeeIdFromUserId - Current employee id is %s' %(ID))
        return ID

      
    def getJobTitleId(self) :
        if self.debug:
            logger.debug('getJobTitleId')
        query = QSqlQuery()
        ID = ""
        query.exec_("select job_Id from hr_employee where user_id = %s" % self.currentUserID)
        while query.next():
            ID = str(query.value(0)) 
        if self.debug:
            logger.debug('getProjectId - project id is %s' %(ID))
        return ID


    def getDepartmentFromUserId(self):
        if self.debug:
            logger.debug('getDepartmentFromUserId')
            logger.debug('Current user id is %s' %(self.currentUserID))
       # newid = ""
        client = erppeek.Client(self.erppeekString, self.sqlDbName , self.erppeekUser , self.erppeekUserPassword)
        #client = erppeek.Client.from_config('atlas')
            #client = erppeek.Client(SERVER, DATABASE, USERNAME, PASSWORD)
            #client = erppeek.Client(connection_string)
        proxyuser = client.model('res.users').browse(int(self.currentUserID))
        department = proxyuser.employee_ids.job_id.name
       # logger.debug('We are printing department now')
       # logger.debug('%s' %(department[0]))
        dpt = department[0]
        #proxyuser = model('res.users').browse(7)
        return dpt
     
     #def getEmployeeIdFromUserId(self):
        #if self.debug:
            #logger.debug('getEmployeeIdFromUserId')
            #logger.debug('Current user id is %s' %(self.currentUserID))
       ## newid = ""
        #client = erppeek.Client(self.erppeekString, self.sqlDbName , self.erppeekUser , self.erppeekUserPassword)
        #proxyuser = client.model('res.users').browse(int(self.currentUserID))
        #department = proxyuser.employee_ids.job_id.name
       ## logger.debug('We are printing department now')
       ## logger.debug('%s' %(department[0]))
        #dpt = department[0]
        ##proxyuser = model('res.users').browse(7)
        #return dpt
            
            

#    def savePartName(self, partName ):
#        if self.debug:
#            logger.debug('Saving new partName')
#        query = QSqlQuery()
#        query.prepare("insert into parts (part_Name)"
#                  "VALUES (:pname)")
#        query.bindValue(":pname", QVariant(partName))
#        query.exec_()
#        if self.debug:
#            print query.lastQuery()
#        if not query.isActive():
#            QMessageBox.warning(self, "Daily time sheet -- Database Error",
#                    "Failed to open database %s" % (query.lastError().text()))

    def dateChanged(self):
        if self.debug:
            logger.debug('date changed UserID is %s' % self.currentUserID)
        self. setDirty()
        if self.debug:
            logger.debug('Filtering with ID')
        #self.hourModel.setFilter(QString("user_id = %1").arg(self.currentUserID))##pg
        #self.hourModel.setFilter("FK_empl_Id = %s" %UserID)
        date = self.calenderDay.selectedDate()
        week, yr = date.weekNumber()
        self.currentDate = date
        self.currentWeek = week
        self.lblDate.setText("   Selected Date : %s-%s-%s   " % (date.day(), date.month(), date.year()))
        newMonth = date.month()
        if not self.currentMonth is  newMonth:
            self.monthChanged = True
            self.currentMonth = newMonth
        else:
            self.monthChanged = False
        self.clearTaskDetails()
        self.buttonEdit.setEnabled(False)
       # self.buttonNew.setEnabled(False)
        self.checkDailyTime()
        #self.loadStatistics()



    def createConnection(self):
        """Create a new connection to the Db
                 using information obtained from the XML file configuration
                """
        QSqlDatabase.removeDatabase("qt_sql_default_connection")
        connection = QSqlDatabase.addDatabase('QPSQL' )
        connection.setHostName(self.sqlDbHost )
        connection.setDatabaseName( self.sqlDbName )
        connection.setUserName( self.sqlDbUser)
        connection.setPassword( self.decodedPassword )
        logger.debug('connection string is  %s -  %s -  %s -  %s' %(self.sqlDbHost, self.sqlDbName, self.sqlDbUser, self.decodedPassword))
        if not connection.open():
            QMessageBox.critical(None,
            self.tr("Error opening connection "),
            self.tr("""Error opening connection - %s""" % connection.lastError().text()),
            QMessageBox.StandardButtons(\
                QMessageBox.Ok))
            return False
        #   print "connection ok"
        return connection

    #def createSettings(self):
        #file = "dbsettings.ini"
        #if self.debug:
            #logger.debug('Saving settings')
        #settings = QSettings(file, QSettings.IniFormat)
        #settings.setValue("sqlDbHost", QVariant("localhost"))
        #settings.setValue("sqlDbName", QVariant("atlas-dev2"))
        #settings.setValue("sqlDbUser", QVariant("postgres"))
        #settings.setValue("sqlDbPasswd", QVariant("o"))
        #settings.setValue("LastUser", QVariant(""))
        #settings.sync()

    def getSettings(self):
        if self.debug:
            logger.debug('get settings')
        file = "dbsettings.ini"
        settings = QSettings(file, QSettings.IniFormat)
        self.sqlDbHost = settings.value("sqlDbHost", "")
        self.sqlDbName = settings.value("sqlDbName", "")
        self.sqlDbUser = settings.value("sqlDbUser", "")
        self.decodedPassword = settings.value("sqlDbPasswd")
        self.erppeekUser = settings.value("erppeekUser", "")
        self.erppeekUserPassword = str(base64.b64decode(settings.value("erppeekUserPassword", "")), encoding='UTF-8')
        self.erppeekString = "http://" + self.sqlDbHost + ":8069"
        
        if self.sqlDbHost == "" or self.sqlDbUser == "" or self.sqlDbName == "":
            logger.debug('Creating new settings file')
            return False
            #self.createSettings()
        else:
            if not str(settings.value("sqlDbPasswd", "")) == "":
               print (str(settings.value("sqlDbPasswd","")))
               self.decodedPassword = str(base64.b64decode(settings.value("sqlDbPasswd", "")), encoding='UTF-8')
            else:
               self.decodedPassword = ""
            return True
       
       

    def insertDataToTable(self, table, values):
        if self.debug:
            logger.debug('insertDataToTable')
        newid = ""
        if table:
            client = erppeek.Client(self.erppeekString, self.sqlDbName , self.erppeekUser , self.erppeekUserPassword)
            #client = erppeek.Client.from_config('atlas')
            proxy = client.model(table)
            newid = proxy.create(values)
            newid = newid.id
        if newid:
            return newid 
        else:
            return False



    def updateDataToTable(self, table, values, id):
        if self.debug:
            logger.debug('updateDataToTable')
        if table:
            client = erppeek.Client(self.erppeekString, self.sqlDbName , self.erppeekUser , self.erppeekUserPassword)
            #client = erppeek.Client.from_config('atlas')
            proxy = client.model('%s'%table)
            proxy.write(id, values)



    def postSaveDailyTime(self):
        if self.debug:
            logger.debug('post saving daily time ')
        self.lblMessage.setText("Saving daily timecard")
        #print "cursor on postSaveDailyTime"
        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
        timeIn = str(self.inLineEdit.text())
        timeOut = str(self.outLineEdit.text())
        date = self.currentDate
        week = self.currentWeek
        pydate = date.toPyDate()
        dt_D = str(pydate.day)
        dt_M = str(pydate.month)
        dt_Y = str(pydate.year)
        dt = "%s-%s-%s"%(dt_Y, dt_M, dt_D)
        dt_in = "%s %s:00" %(dt, timeIn)
        dt_out = "%s %s:00" %(dt, timeOut)
        employeeID = self.getEmployeeIdFromUserId()
        if   self.noBreakCheckBox.isChecked() :
            lBreak = True
        else:
            lBreak = False
        totTime = str(self.leWorkedHours.text())
        self.progressMessage.setVisible(True)
        self.progressMessage.setValue(0)
        query = QSqlQuery()
        if not self.timeCardInDatabase:
            if self.debug:
                logger.debug("Trying oerp-saving daily time ")
            try:
                timecard_data = {
                    'name': dt,
                    'time_in': timeIn,
                    'time_out': timeOut,
                    'worked_hours': totTime,
                    'nolunchbreak': lBreak,
                    'week_no':week,
                    'user_id': str(self.currentUserID)}   
                attendance_in_data = {
                    'name': dt_in,
                    'action': "sign_in",
                    'employee_id': str(employeeID)} 
                attendance_out_data = {
                    'name': dt_out,
                    'action': "sign_out",
                    'employee_id': str(employeeID)}
                timecard_id = self.insertDataToTable('hr.timecard', timecard_data)
                if self.debug:
                     logger.debug("Time card id is %s " %timecard_id)
                     logger.debug("In data is %s " %attendance_in_data)
                     logger.debug("Out data is %s " %attendance_out_data)
                attendance_in_id = self.insertDataToTable('hr.attendance', attendance_in_data)
                if self.debug:
                     logger.debug("Atten in id is %s " %attendance_in_id)
                attendance_out_id = self.insertDataToTable('hr.attendance', attendance_out_data)
                if self.debug:
                     logger.debug("Atten out id is %s " %attendance_out_id)
                self.leTimeCardId.setText(str(timecard_id))
                self.progressMessage.setValue(30)
            except Exception as e:
                QMessageBox.warning(self, "Daily time sheet-Database Error",
                    "Error saving data to server, %s" %(e))
        else:
            if self.debug:
                logger.debug("Updating timecard")
            self.lblMessage.setText("Updating Timecard")
            query.prepare("UPDATE hr_timecard SET time_in = :in ,time_out = :out, nolunchBreak = :lBreak, "
            "worked_hours = :totHour, week_no = :we WHERE name = :dt_dt AND user_id = :id")
            query.bindValue(":in", QVariant(timeIn))
            query.bindValue(":out", QVariant(timeOut))
            query.bindValue(":lBreak", QVariant(lBreak))
            query.bindValue(":totHour", QVariant(totTime))
            query.bindValue(":dt_dt", QVariant(dt))
            query.bindValue(":id", QVariant(self.currentUserID))
            query.bindValue(":we", QVariant(week))
            self.progressMessage.setValue(50)
            query.exec_()
            if not query.isActive():
                QMessageBox.warning(self, "Daily time sheet -- Database Error",
                        "Failed to open database %s" % (query.lastError().text()))
                self.lblMessage.setText("Saving data failed")
                self.progressMessage.setValue(0)
                self.chartDrawn = False
                QApplication.restoreOverrideCursor()
            else:
                self.progressMessage.setValue(80)
                self.chartDrawn = False
            #self.monthGraphDataForMatplot()
        self.timeCardInDatabase = True
        self.inTimeText = self.inLineEdit.text()
        self.outTimeText = self.outLineEdit.text()
        self._setMonthFilter(date, 1)
        self.progressTimer.start(2000)
        self.progressMessage.setValue(100)
        self.updateUi()
        self.frameDailyTimeCardButtons.setEnabled(True)
        self.hourView.setEnabled(True)
        #self.monthGraphDataForMatplot()
        self.lblMessage.setText("Timecard saved")
        #print "cursor off  postSaveDailyTime"
        QApplication.restoreOverrideCursor()




    def saveDailyTime(self):
        if self.debug:
            logger.debug('saving daily time-pre ')
        if not self.callingFromProgram:
            inTime = self.inLineEdit.text()
            outTime = self.outLineEdit.text()
            if not len(inTime) == 0 and not len(outTime) == 0 :

                date = self.currentDate
                pydate = date.toPyDate()
                dt_D = str(pydate.day)
                dt_M = str(pydate.month)
                dt_Y = str(pydate.year)
                #self.oldTotalTime = unicode(self.totLineEdit.text())
                self.calculateDailyWorkedHours()
                totTime = self.leWorkedHours.text()
                if  len(totTime) == 0:
                    totTime = 0;

                if self.debug:
                    logger.debug('totTime is %s' % (totTime))
                    logger.debug('totTime from widget is  %s' % (self.leWorkedHours.text()))

                totTime = float(totTime)
                if totTime > 12:
                    if not totTime == self.oldTotalTime:
                        reply = QMessageBox.question(None,
                            "Please check hours worked",
                            "It appears you worked a total of <b> %s hours on %s-%s-%s.</b><br>Do you want to save the time ? <br> Or you have made a mistake in entering <br> In Time or Out time: In and Out times required to be <br> entered in 24 hour format." % (totTime, dt_D, dt_M, dt_Y),
                            QMessageBox.StandardButtons(\
                                QMessageBox.Cancel | \
                                QMessageBox.No | \
                                QMessageBox.Yes))
                        # If they say yes, go ahead and do it.
                        if reply == QMessageBox.Yes:
                            self.postSaveDailyTime()
                        else:                       
                            self.inLineEdit.setFocus()
                            self.inLineEdit.selectAll()
                    #logger.debug('Old time is  %s' % (self.oldTotalTime))
                    #lo      gger.debug('Setting Old time to  %s' % (totTime))
                    self.oldTotalTime = totTime
                else:
                    self.postSaveDailyTime()
        else:
            logger.debug('Called from within program')



        #TODO
#    def loadStatistics(self):
#        date = self.calenderDay.selectedDate()
#        query = QSqlQuery()
#        self.lbl_hours_week.setText("0 Hrs")
#        self.lbl_hours_month.setText("0 Hrs")
#        self.lbl_hours_year.setText("0 Hrs")
#        #query.exec_("select time_In,time_Out,isDataEntered,lunchBreak,totHours,inAmPm,outAmPm from timecard  where FK_empl_Id = '%s' and worked_Day = %s and worked_Month = %s and worked_Year = %s"  % (self.currentUserID, date.day(), date.month(), date.year()))
#        query.exec_("SELECT sum(worked_hours)+sum(overtime_hours) as hours FROM smeclient_work WHERE date = '%s' AND user_id = %s " % (dt, self.currentUserID))
#        while query.next():
#            self.lbl_hours_week.setText("%s Hrs" % (QString(query.value(0).toString())))
#            #self.lbl_hours_week.setText(QString(query.value(0).toString()))
#
#        query.exec_("SELECT SUM(worked_hours) FROM hr_timecard  WHERE  worked_Month=%s AND user_id =%s AND worked_Year =%s" % (date.month(), self.currentUserID, date.year()))
#        while query.next():
#            self.lbl_hours_month.setText("%s Hrs" % (QString(query.value(0).toString())))
#            #self.lbl_hours_month.setText(QString(query.value(0).toString()))
#
#        query.exec_("SELECT SUM(totHours) FROM management.timecard  WHERE  worked_Year=%s AND FK_empl_Id =%s" % (date.year(), self.currentUserID))
#        while query.next():
#            self.lbl_hours_year.setText("%s Hrs" % (QString(query.value(0).toString())))
#           # self.lbl_hours_year.setText(QString(query.value(0).toString()))
#


    #TODO
    def deleteDailyTime(self):
        if self.debug:
            logger.debug('deleteDailyTime ')
        date = self.currentDate
       # print "cursor on deleteDailyTime"
        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
        self.lblMessage.setText("Deleting Timecard for %s" % date)
        query = QSqlQuery()
        date = self.currentDate
        pydate = date.toPyDate()
        dt_D = str(pydate.day)
        dt_M = str(pydate.month)
        dt_Y = str(pydate.year)
        dt = "%s-%s-%s"%(dt_Y, dt_M, dt_D)
        if  self.timeCardInDatabase:
            res = QMessageBox.question(None,
                self.tr("Do you want to delete the details"),
                self.tr("""Do you want to delete the details for  %s""" % date),
                QMessageBox.StandardButtons(\
                    QMessageBox.Cancel | \
                    QMessageBox.No | \
                    QMessageBox.Yes),
                QMessageBox.No)
            if res == QMessageBox.Yes:
                query.prepare("DELETE from hr_timecard where user_id=:id and name=:dt ")
                query.bindValue(":id", QVariant(self.currentUserID))
                query.bindValue(":dt", QVariant(dt))
                query.exec_()
        if self.debug:
            print (query.lastQuery())
        if not query.isActive():
            QMessageBox.warning(self, "Daily time sheet -- Database Error",
                    "Failed to open database %s" % (query.lastError().text()))
        else:
            #if not progressForm:
            self.chartDrawn = False
           # self.monthGraphDataForMatplot()
            self.timeCardInDatabase = False
        self.inTimeText = self.inLineEdit.text()
        self.outTimeText = self.outLineEdit.text()
        self._setMonthFilter(date, 1)
        self.updateUi()
        #print "cursor off  deleteDailyTime"
        QApplication.restoreOverrideCursor()



    def updateUi(self, enable=False):
        #print "Updating ui"
        if self.jobDataSaved:
            self.btSaveTaskWork.setEnabled(False)
            #self.buttonEdit.setEnabled(False)
        else:
            self.btSaveTaskWork.setEnabled(True)
        if self.timeCardInDatabase:
            if not(self.inLineEdit.text() == "" or \
                          self.outLineEdit.text() == ""):
                self.editButton.setEnabled(True)
                self.deleteTimeButton.setEnabled(True)
                self.inLineEdit.setEnabled(False)
                self.outLineEdit.setEnabled(False)
                self.buttonNew.setEnabled(True)
                self.btDeleteTaskWork.setEnabled(False)
            else:
                self.editButton.setEnabled(False)
                self.deleteTimeButton.setEnabled(False)
                self.inLineEdit.setEnabled(True)
                self.outLineEdit.setEnabled(True)
                self.buttonNew.setEnabled(True)
                self.btDeleteTaskWork.setEnabled(False)
        else:
            if not(self.inLineEdit.text()== "" and \
                           self.outLineEdit.text()== ""):
                self.inLineEdit.setFocus()
            self.editButton.setEnabled(False)
            self.deleteTimeButton.setEnabled(False)
            self.inLineEdit.setEnabled(True)
            self.outLineEdit.setEnabled(True)
            self.buttonNew.setEnabled(True)
            self.btDeleteTaskWork.setEnabled(False)
        if enable:
            self.editButton.setEnabled(False)
            self.deleteTimeButton.setEnabled(False)
            self.inLineEdit.setEnabled(True)
            self.outLineEdit.setEnabled(True)
            self.noBreakCheckBox.setEnabled(True)


    def reformatCalendarPage(self, date):
        #if self.debug:
            #logger.debug('reformatting calendar')
        mayFirstFormat = QTextCharFormat()
        #color = QColor(255, 0, 0, 127) #red
        color = QColor(184, 255, 197, 255)
        #mayFirstFormat.setForeground(Qt.red)
        mayFirstFormat.setBackground(color)
        #date = QDate(2008, 10, 20)
        self.calenderDay.setDateTextFormat(date, mayFirstFormat)


#    def getUserIDForChart(self,  user):
#        query = QSqlQuery()
#        query.exec_("SELECT empl_Id,FK_empl_Deptmnt FROM employee "
#                "where empl_First_Name = '%s'" % user)
#        while query.next():
#            ID = ""
#            ID = QString(query.value(0).toString())
#            #departmentId = QString(query.value(1).toString())
#            #self.currentUserID = ID
#            return ID
#            #print ID
#            if self.debug:
#                logger.debug('user ID is %s' % ID)
#            return ID

    def checkWorkExistsInOpenerp(self,  workid):
        if self.debug:
                logger.debug('checkWorkExistsInOpenerp')
        ID = False
        query = QSqlQuery()
        query.exec_("SELECT * FROM project_task_work WHERE id = '%s'" % workid)
        if self.debug:
                logger.debug('Checking if the work exists  %s' % workid)
        while query.next():
            ID = True
            if self.debug:
                logger.debug('There is work in the db')
        return  ID


    def checkJobNoExists(self, jobNo):
        if self.debug:
                logger.debug('checkJobNoExists')
        ID = False
        query = QSqlQuery()
        query.exec_("SELECT * FROM project_project WHERE job_No = '%s'" % jobNo)
        if self.debug:
                logger.debug('Checking if the job exists  %s' % jobNo)
        while query.next():
            ID = True
            if self.debug:
                logger.debug('There is job in the db')
        return  ID

    def checkTaskNameExists(self,  taskName):
        if self.debug:
                logger.debug('checkTaskNameExists')
        ID = False
        query = QSqlQuery()
        query.exec_("SELECT id FROM project_task WHERE name = '%s'" % taskName)
        if self.debug:
                logger.debug('Checking if the task exists  %s' % taskName)
        while query.next():
            ID = True
            #self.leTaskId.setText(QString(query.value(0).toString()))
            if self.debug:
                logger.debug('There is task in the db')
        return  ID

    def reInitTable(self):
       #print "Re initializing table"
       self.hourModel.select()

    def checkDailyTime(self):
        if self.debug:
            logger.debug('checking daily time ')
        date = self.calenderDay.selectedDate()
        self.callingFromProgram = True
        #for checking
        self.inLineEdit.setText("")
        self.outLineEdit.setText("")
        self.leWorkedHours.setText("0.00")
        self.leDataEntered.setText("0.00")
        self.leRemainingHours.setText("0.00")
        self.lbl_hours_daily.setText("0.00 Hrs")
        self.leTimeCardId.setText("")
        self.noBreakCheckBox.setChecked(False)
       # self.noBreakCheckBox.setEnabled(False)
        self.frameDailyTimeCardButtons.setEnabled(False)
        self.hourView.setEnabled(False)
        self.timeCardInDatabase = False
        self.dailyTimeCalculated = False
        self.jobDataSaved = False
        if not self.checkConnection():
            self.createConnection()
            self.reInitTable()
        query = QSqlQuery()
        query.exec_("SELECT time_in, time_out, worked_hours, nolunchbreak, id FROM hr_timecard where user_id = '%s' and name='%s-%s-%s'" % (self.currentUserID, date.year(),date.month(),date.day()))

        while query.next():
           # print "Query is successful"
            self.inLineEdit.setText(query.value(0))
            self.outLineEdit.setText(query.value(1))
            self.timeCardInDatabase = True
            self.jobDataSaved = True
            self.frameDailyTimeCardButtons.setEnabled(True)
            self.hourView.setEnabled(True)
            self.leTimeCardId.setText(str(query.value(4)))
            self.leWorkedHours.setText(str(query.value(2)))
            self.lbl_hours_daily.setText("%s Hrs" % (query.value(2)))
            self.getWorkedHours()
            if query.value(3) == "true":
                self.noBreakCheckBox.setChecked(True)
            else:
                self.noBreakCheckBox.setChecked(False)
            self.deleteTimeButton.setEnabled(False)
            self.editButton.setEnabled(True)

        self.updateUi()
        self.inTimeText = self.inLineEdit.text()
        self.outTimeText = self.outLineEdit.text()
        self.callingFromProgram = False



    def calculateDailyWorkedHours(self):
        if self.debug:
            logger.debug('calculating hours ')
        self.dailyTimeCalculated = False
        inTime = self.inLineEdit.text()
        outTime = self.outLineEdit.text()
#        #splits time input into usable chunks
        if not len(inTime) == 0 and not len(outTime) == 0 :
            if ((":" in inTime)  or  ("." in inTime)or ("," in inTime)):
                if self.debug:
                    logger.debug('intime does  have delimiters')
                inTime = inTime.replace(",", ":")
                inTime = inTime.replace(".", ":")
                if self.debug:
                    logger.debug('intime is %s' % inTime)
            else:
                if self.debug:
                    logger.debug('intime does not have delimiters')
                inTime = ":".join((inTime, "00"))
                if self.debug:
                    logger.debug('intime does not have delimiters %s' % inTime)
            if ((":" in outTime)  or  ("." in outTime)or ("," in outTime)):
                outTime = outTime.replace(",", ":")
                outTime = outTime.replace(".", ":")
                if self.debug:
                    logger.debug('outtime is %s' % outTime)
            else:
                outTime = ":".join((outTime, "00"))
            if self.debug:
                logger.debug('intime is %s and outTime is %s' % (inTime , outTime))
            if inTime.endswith(":"):
                inTime = "".join((inTime, "00"))
            if outTime.endswith(":"):
                outTime = "".join((outTime, "00"))
            self.inLineEdit.setText(inTime)
            self.outLineEdit.setText(outTime)
            #startSplit = string.split(inTime, ":")
           # endSplit = string.split(outTime, ":")
            
            startSplit = inTime.split(":")
            endSplit = outTime.split(":")
            #converts strings to usable integers
            sh = int(startSplit[0])
            sm = int(startSplit[1])
            eh = int(endSplit[0])
            em = int(endSplit[1])
            # Calculate minute-of-day
            smin = sh * 60 + sm
            emin = eh * 60 + em
            # Delta time in  minutes
            dmin = emin - smin
            if dmin < 0: dmin += 1440  # Wrapped around midnight?
            if  not self.noBreakCheckBox.isChecked() :
                dmin -= 30
            # Delta time in hours
            dtime = dmin / 60.
            self.leWorkedHours.setText(str(dtime))
 #           self.addDailyTimeButton.setEnabled(True)
            self.noBreakCheckBox.setEnabled(True)
            self.dailyTimeCalculated = True

    def addWork(self):
        if self.debug:
            logger.debug('adding new row to work ')
        self.btSaveTaskWork.setEnabled(True)
        #logger.debug('print 0')
        row = self.hourModel.rowCount()
        #logger.debug(row)
        #logger.debug('print 1')
        date = self.calenderDay.selectedDate()
        self.hourModel.insertRow(row)
        index = self.hourModel.index(row, EMPLID)
        data = self.currentUserID
        self.hourModel.setData(index, QVariant(data) , (Qt.EditRole))
        index = self.hourModel.index(row, DATE)
        data = date
        self.hourModel.setData(index, QVariant(data) , (Qt.EditRole))
        index = self.hourModel.index(row, 1)
        self.hourView.setCurrentIndex(index)
        self.jobDataSaved = False
        index = self.hourModel.index(row, JOBNO)
        self.hourView.setCurrentIndex(index)
        self.buttonEdit.setEnabled(False)
        self.buttonNew.setEnabled(False)
        self.hourView.edit(index)
        self.newRow = True
        #self.hourView.edit(index)

    def setDataModel(self):
        index = self.hourView.currentIndex()
        if self.debug:
            print ("Index %s" % index)

    def EnbleSave(self):
        #print "cursor on EnbleSave"
        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
        if self.debug:
            logger.debug('Enablng save button ')
        self.btSaveTaskWork.setEnabled(True)
        index = self.hourView.currentIndex()
        #if self.debug:
            #logger.debug('Index %s' )%index
            #print index
        row = self.hourView.currentIndex().row()
        index = self.hourModel.index(row, JOBNO)
        if self.debug:
            logger.debug('Data from EnbleSave is')
            logger.debug(str(self.hourModel.data(index, (Qt.EditRole))))
            #message = "Part Name"
        JOB_NO = str(self.hourModel.data(index, (Qt.EditRole)))
        if not JOB_NO =="":
            self.newRow = True
        if  JOB_NO == "" or len(JOB_NO) < 5:
            if  not self.newRow:
                QMessageBox.critical(None,
                    self.tr("Invalid data for Job no"),
                    self.tr("""Please enter valid data  for Job No""" ),
                    QMessageBox.StandardButtons(\
                        QMessageBox.Ok))
                self.newRow = False
            #print "cursor off  EnbleSave"
            QApplication.restoreOverrideCursor()
            return
        else:
            jobExists = self.checkJobNoExists(JOB_NO)
            if not jobExists:
                self.progressMessage.setVisible(True)
                self.progressMessage.setValue(0)
                self.message.setText("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
    "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
    "p, li { white-space: pre-wrap; }\n"
    "</style></head><body style=\" font-family:\'MS Shell Dlg 2\'; font-size:8.25pt; font-weight:400; font-style:normal;\">\n"
    "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:8pt;\"><span style=\" font-size:9pt; font-weight:600; color:#aa0000;\">Added new job in the database</span></p>\n"
    "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt; font-weight:600; color:#aa0000;\"><span style=\" font-size:8pt; font-weight:400; color:#000000;\">Job No %s is added to the database.</p></body></html>" %self.hourModel.data(index, (Qt.EditRole)))
                self.fmMessage.setVisible(True)
                self.lblMessage.setText("Creating new Job in database")
                self.createNewJobInOpenerp(JOB_NO)
                self.lblMessage.setText("New job created in database")
                self.progressMessage.setValue(30)
                self.messageTimer.start(3000)
                self.lblMessage.setText("Creating default tasks  in database")
                self.createDefaultTasksInOpenerp(JOB_NO)
                self.lblMessage.setText("Default tasks  created in database")
                self.progressMessage.setValue(100)
                self.progressTimer.start(2000)
            self.delegate.setJobNo(str(self.hourModel.data(index, (Qt.EditRole))))
        #print "cursor off  EnbleSave"
        QApplication.restoreOverrideCursor()

    def checkConnection(self):
        #print "Checking connection"
        self.lblMessage.setText("Checking connection to database")
      #  print "cursor on checkConnection"
        #QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
        query = QSqlQuery()
        query.exec_("select login,id,partner_id from res_users where active=True order by login ASC")
        if query.isActive():
           # QApplication.restoreOverrideCursor()
            return True
           # print "Database connection is ok"
        else:
           # print "Database connection is not ok"
            return False
      #  print "cursor off  checkConnection"
        


    def submit(self):
        if self.debug:
            logger.debug('submit')
        #print "cursor on submit"
        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
        self.btSaveTaskWork.setEnabled(False)
        self.buttonEdit.setEnabled(False)
        self.btDeleteTaskWork.setEnabled(False)
        self.buttonNew.setEnabled(False)
        self.lblMessage.setText("Saving work details to database")
        ###
        id = ""
        oerpWorkId = ""
        normalHours = 0.0
        overtimeHours = 0.0
        unAttendedHours = 0.0
        jobNo = ""
        projectId = ""
        taskId = ""
        taskName = "Select task"
        workSummery = ""
        workInOerp = False
        if self.debug:
            logger.debug('Saving new row to work')
        row = self.hourView.currentIndex().row()
        index = self.hourModel.index(row, WORKID)
        id = str(self.hourModel.data(index, (Qt.EditRole)))
        print ("printing id")
        print (id)
        index = self.hourModel.index(row, ERPWORKID)
        oerpWorkId = str(self.hourModel.data(index, (Qt.EditRole)))
        print ("Printing oerpWorkId from start")
        print (oerpWorkId)
        ###
        index = self.hourModel.index(row, JOBNO)
        print("Printing job no")
        print(str(self.hourModel.data(index, (Qt.EditRole))))
        if  (str(self.hourModel.data(index, (Qt.EditRole))) == ""):
            QMessageBox.critical(None,
                self.tr("Incomplete data for Job no"),
                self.tr("""Please enter mandatory data  for Job No""" ),
                QMessageBox.StandardButtons(\
                    QMessageBox.Ok))
            #print "cursor off submit"
            self.btSaveTaskWork.setEnabled(True)
            QApplication.restoreOverrideCursor()
            return
        else:
            jobExists = self.checkJobNoExists(str(self.hourModel.data(index, (Qt.EditRole))))
            if not jobExists:
               self.message.setText("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'MS Shell Dlg 2\'; font-size:8.25pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:8pt;\"><span style=\" font-size:9pt; font-weight:600; color:#aa0000;\">Added new job in the database</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-size:9pt; font-weight:600; color:#aa0000;\"><span style=\" font-size:8pt; font-weight:400; color:#000000;\">Job No %s is added to the database.</p></body></html>" %str(self.hourModel.data(index, (Qt.EditRole))))
               self.fmMessage.setVisible(True)
               self.createNewJobInOpenerp(str(self.hourModel.data(index, (Qt.EditRole))))
               self.messageTimer.start(2000)
            else:
                jobNo = str(self.hourModel.data(index, (Qt.EditRole)))
        index = self.hourModel.index(row, TASK)
        if  (str(self.hourModel.data(index, (Qt.EditRole))) == ""):
            taskName = '%s-'%(jobNo)
            self.hourModel.setData(index, QVariant(taskName) , (Qt.EditRole))
        else:
            taskName = str(self.hourModel.data(index, (Qt.EditRole)))
            checkTask = taskName[:5]
           # print checkTask
            index = self.hourModel.index(row, JOBNO)
            jobNo = str(self.hourModel.data(index, (Qt.EditRole)))
            if not checkTask == jobNo:
                QMessageBox.critical(None,
                self.tr("Invalid task name, Tasks must start with Job No"),
                self.tr("""Tasks must start with the same Job No you entered. If there is tasks already in the system, please select the appropriate one from the drop down, To create a new task, please enter task name starting with Job No , Please change your task to include Job No at the start. """),
                QMessageBox.StandardButtons(\
                    QMessageBox.Ok))
                #print "cursor off submit"
                QApplication.restoreOverrideCursor()
                self.btSaveTaskWork.setEnabled(True)
                return
        index = self.hourModel.index(row, TASK)
        taskExists = self.checkTaskNameExists(str(self.hourModel.data(index, (Qt.EditRole))))
        if not taskExists:
            taskId = self.createNewTaskInOpenerp(jobNo,  taskName)
            self.leTaskId.setText(str(taskId))
          # self.messageTimer.start(100)
        else:
            taskId = self.getTaskIdFromTaskName(taskName)

        index = self.hourModel.index(row, WSUMMARY)
        if  (str(self.hourModel.data(index, (Qt.EditRole)))) == "" :
            workSummery = ""
            str(self.hourModel.setData(index, QVariant(workSummery) , (Qt.EditRole)))
        else:
            workSummery = str(self.hourModel.data(index, (Qt.EditRole)))
        index = self.hourModel.index(row, UHRS)
        unAttendedHours = str(self.hourModel.data(index, (Qt.EditRole)))
        print ("Unattended")
        print (unAttendedHours)
        if  unAttendedHours == '0.0' or unAttendedHours == "":
           # print "Data is empty for ut"
            unAttendedHours = 0.0
            unAttendedHours = float(unAttendedHours)
            self.hourModel.setData(index, QVariant(unAttendedHours) , (Qt.EditRole))
        
        unAttendedHours = float(unAttendedHours)
        index = self.hourModel.index(row, OVERTIME)
        overtimeHours = str(self.hourModel.data(index, (Qt.EditRole)))
        print ("overtimeHours")
        print (overtimeHours)
        if  overtimeHours == '0.0' or overtimeHours == "":
           # print "Data is empty for ot"
            overtimeHours = 0.0
            overtimeHours = float(overtimeHours)
            self.hourModel.setData(index, QVariant(overtimeHours) , (Qt.EditRole))
        index = self.hourModel.index(row, HOUR)
        normalHours = str(self.hourModel.data(index, (Qt.EditRole)))
        print ("normalHours")
        print (normalHours)
        if  normalHours == '0.0' or normalHours == "":
            #print "Data for hours is 0"
            normalHours = 0.0
            normalHours = float(normalHours)
            self.hourModel.setData(index, QVariant(normalHours) , (Qt.EditRole))
        intData = float(normalHours)
       # print intData is
        print ("data is ")
        print (intData)
        if intData == 0.0 and overtimeHours == 0.0:
            QMessageBox.critical(None,
                self.tr("Invalid Worked Hours or overtime hours "),
                self.tr("""You must enter data for either normal ours or overtime hours field, <br>  You can not leave both fields empty or they can not be zero"""),
                QMessageBox.StandardButtons(\
                    QMessageBox.Ok))
            #print "cursor off submit"
            self.btSaveTaskWork.setEnabled(True)
            QApplication.restoreOverrideCursor()
            return
        if  (intData > 8):
            QMessageBox.critical(None,
                self.tr("You can not enter more than 8 Worked Hours "),
                self.tr("""You can not enter more than 8 hours in  Worked Hours field, Please enter correct hours  by splitting in to Overtime hours"""),
                QMessageBox.StandardButtons(\
                    QMessageBox.Ok))
            #print "cursor off submit"
            self.btSaveTaskWork.setEnabled(True)
            QApplication.restoreOverrideCursor()
            return

       # logger.debug('in  calculateTotalHoursEntered')
        row = self.hourView.currentIndex().row()
        totHours = 0.0
        totOHrs = 0.0
        totWorkedHrs = self.leWorkedHours.text()
        if  len(totWorkedHrs) == 0.0:
                totWorkedHrs = 0.0
        totWorkedHrs = float(totWorkedHrs)
        for t in range(self.hourModel.rowCount()):
            index = self.hourModel.index(t, HOUR)
            hrs = str(self.hourModel.data(index, (Qt.EditRole)))
            if  len(hrs) == 0 or hrs == "0.0":
                hrs = 0.0
            hrs = float(hrs)
            index = self.hourModel.index(t, OVERTIME)
            oHrs = str(self.hourModel.data(index, (Qt.EditRole)))
            if  len(oHrs) == 0 or hrs == "0.0":
                oHrs = 0.0

            oHrs = float(oHrs)
            totHours = totHours + hrs
            totOHrs = totOHrs + oHrs
        totDataEntered = totHours + totOHrs
        if totHours > 8:
            QMessageBox.critical(None,
                self.tr("You can not enter more than 8 Worked Hours "),
                self.tr("""You can enter a total of  8 hours in  Worked Hours, Please enter correct hours  by splitting in to Overtime hours"""),
                QMessageBox.StandardButtons(\
                    QMessageBox.Ok))
            #print "cursor off submit"
            self.btSaveTaskWork.setEnabled(True)
            QApplication.restoreOverrideCursor()
            return
        if totDataEntered > totWorkedHrs :
            QMessageBox.critical(None,
                self.tr("You can not enter more than Worked Hours "),
                self.tr("""You can enter only up to %s Hrs,  Worked Hours and Overtime Hours combined. If you want to enter more hours, Please correct IN and Out times  to reflect the Total hours worked """ % totWorkedHrs),
                QMessageBox.StandardButtons(\
                    QMessageBox.Ok))
            #print "cursor off submit"
            self.btSaveTaskWork.setEnabled(True)
            QApplication.restoreOverrideCursor()
            return

            if self.debug:
                print ("You need to enter data")
        if self.hourModel.submitAll():
            date = self.currentDate
            pydate = date.toPyDate()
            dt_D = str(pydate.day)
            dt_M = str(pydate.month)
            dt_Y = str(pydate.year)
            dt = "%s-%s-%s"%(dt_Y, dt_M, dt_D)
            self.jobDataSaved = True
            self.progressMessage.setVisible(True)
            self.progressMessage.setValue(0)
            self.chartDrawn = False
            self.monthGraphDataForMatplot()
            self.getWorkedHours()
            self.progressMessage.setValue(50)
            self.btSaveTaskWork.setEnabled(False)
            self.buttonEdit.setEnabled(False)
            self.btDeleteTaskWork.setEnabled(False)
            self.buttonNew.setEnabled(True)
            
            #row = self.hourView.currentIndex().row()
            #index = self.hourModel.index(row, WORKID)
            id = self.getWorkId()
            #print "New id is"
            #print id
           # if workInOerp:
            projectId = self.getProjectIdFromJobNo(jobNo)
            oerpWorkData = {
                            'name': str(workSummery), 
                            'hours': str(normalHours), 
                            'user_id': str(self.currentUserID), 
                            'date': dt, 
                            'job_no': str(projectId),  
                            'overtime_hours': str(overtimeHours), 
                            'unattended_hours': str(unAttendedHours), 
                            'task_id': str(taskId)
                            }
            print("Oerpdata")
            print (oerpWorkData)
            if oerpWorkId == "" or oerpWorkId == "0":
                #create new data to project.task.work
                oerpWorkId = self.insertDataToTable('project.task.work', oerpWorkData) 
               # print "Oerp work id is"
                #print intId
                #oerpWorkId = intId.id
                #print oerpWorkId
                #print test
                work_data = {
                            'name': str(jobNo),
                            'task': str(taskName),
                            'work_summary': str(workSummery),
                            'worked_hours': str(normalHours),
                            'overtime_hours': str(overtimeHours),
                            'unattended_hours': str(unAttendedHours),
                            'date': dt,
                            'user_id': str(self.currentUserID), 
                            'workid': str(oerpWorkId)
                            }
                print("Workdata")
                print (work_data)
                self.leOerpWorkId.setText(str(oerpWorkId))
                #update smeclient.work to include oerpworkid
                self.setOerpWorkIdInDatabase(id, oerpWorkId)
                #self.hourModel.refresh()
                #self.updateDataToTable('smeclient.work', work_data, id)
              #pass
            else:
                workInOerp = self.checkWorkExistsInOpenerp(oerpWorkId)
                if workInOerp:
                    if self.debug:
                        logger.debug("------------------------------------------------------------")
                        logger.debug("Updating oerp work id")
                    work_data = {
                            'name': str(jobNo),
                            'task': str(taskName),
                            'work_summary': str(workSummery),
                            'worked_hours': str(normalHours),
                            'overtime_hours': str(overtimeHours),
                            'unattended_hours': str(unAttendedHours),
                            'date': dt,
                            'user_id': str(self.currentUserID), 
                            'workid': str(oerpWorkId)
                            }   
                    self.leOerpWorkId.setText(str(oerpWorkId))
                    try:
                        client = erppeek.Client(self.erppeekString, self.sqlDbName , self.erppeekUser , self.erppeekUserPassword)
                        #client = erppeek.Client.from_config('atlas')
                        proxy = client.model('project.task.work')
                        if self.debug:
                            logger.debug("------------------------------------------------------------")
                            logger.debug("oerpWorkId is")
                            logger.debug(oerpWorkId)
                            logger.debug(oerpWorkData)
                        proxy.write(int(oerpWorkId), oerpWorkData)
                    except Exception as e:
                        QApplication.restoreOverrideCursor()
                        QMessageBox.warning(self, "Daily time sheet-Database Error",
                                "Error saving data to server, %s" %(e))
            self.progressMessage.setValue(100)
            self.progressTimer.start(2000)
            self.reInitTable()
            self.lblMessage.setText("Work details saved to database")
            QApplication.restoreOverrideCursor()
        #    app.restoreOverrideCursor()
        else:
            self.jobDataSaved = False
            QApplication.restoreOverrideCursor()
            if self.debug:
                logger.debug("Error setData %s" % (self.hourModel.lastError()).text())
            QMessageBox.warning(None,
                self.tr("Could not save data"),
                self.tr("Failed to enter  data with error %s" % (self.hourModel.lastError()).text()),
                QMessageBox.StandardButtons(\
                    QMessageBox.Ok))
            self.btSaveTaskWork.setEnabled(True)


    def clearTaskDetails(self):
        self.leTask.setText("")
        self.leProjectId.setText("")
        self.leWorkSummery.setText("")
        self.leWorkId.setText("")
        self.leTaskId.setText("")
        self.leJob_no.setText("")
        self.leTimeSpent.setText("")
        self.leOt.setText("")
        self.leUt.setText("")
        self.leOerpWorkId.setText("")


    def enableDeleteWork(self):
        index = self.hourView.currentIndex()
        if  index.isValid():
            row = self.hourView.currentIndex().row()
            #workid
            index = self.hourModel.index(row, 0)
            t0 =  str(self.hourModel.data(index, (Qt.EditRole)))
            self.leWorkId.setText(str(self.hourModel.data(index, (Qt.EditRole))))
            #jobNo
            index = self.hourModel.index(row, 1)
            t1 =  self.hourModel.data(index, (Qt.EditRole))
            self.leJob_no.setText(str(self.hourModel.data(index, (Qt.EditRole))))
            projectId = self.getProjectIdFromJobNo(t1)
            self.leProjectId.setText(projectId)
            #Task
            index = self.hourModel.index(row, 2)
            t2 =  str(self.hourModel.data(index, (Qt.EditRole)))
            self.leTask.setText(str(self.hourModel.data(index, (Qt.EditRole))))
            taskId =  self.getTaskIdFromTaskName(t2)
            self.leTaskId.setText(taskId)
            #work summery
            index = self.hourModel.index(row, 3)
            self.leWorkSummery.setText(str(self.hourModel.data(index, (Qt.EditRole))))
            t3 =  str(self.hourModel.data(index, (Qt.EditRole)))
            #worked hours
            index = self.hourModel.index(row, 4)
            t4 =  str(self.hourModel.data(index, (Qt.EditRole)))
            self.leTimeSpent.setText(str(self.hourModel.data(index, (Qt.EditRole))))
            #OT
            index = self.hourModel.index(row, 5)
            t5 =  str(self.hourModel.data(index, (Qt.EditRole)))
            self.leOt.setText(str(self.hourModel.data(index, (Qt.EditRole))))
            #ut
            index = self.hourModel.index(row, 6)
            self.leUt.setText(str(self.hourModel.data(index, (Qt.EditRole))))
            t6 =  str(self.hourModel.data(index, (Qt.EditRole)))
            #OerpWorkId
            index = self.hourModel.index(row, 13)
            self.leOerpWorkId.setText(str(self.hourModel.data(index, (Qt.EditRole))))
            t7 =  str(self.hourModel.data(index, (Qt.EditRole)))

        if self.debug:
            logger.debug("Data is t0 %s,t1 %s,t2 %s,t3 %s,t4 %s,t5 %s,t6 %s ,t7 %s" %(t0, t1, t2, t3, t4, t5, t6,  t7))
        index = self.hourView.currentIndex()
        if  index.isValid():
            self.btDeleteTaskWork.setEnabled(True)
            self.buttonEdit.setEnabled(True)
        else:
            self.btDeleteTaskWork.setEnabled(False)
            self.buttonEdit.setEnabled(False)
            self.buttonNew.setEnabled(True)

    def deleteWork(self):
        if self.debug:
                logger.debug("deleteWork")
        index = self.hourView.currentIndex()
        id = self.leOerpWorkId.text()
        if not index.isValid():
            return
        QSqlDatabase.database().transaction()
        msg = "This will<font color=red> Delete </font> worked  hours record from the system, Do you want to proceed"
        msg += "?"
        if QMessageBox.question(self, "Delete worked hours from the system", msg,
                QMessageBox.Yes | QMessageBox.No) == QMessageBox.No:
            QSqlDatabase.database().rollback()
            return
        self.hourModel.removeRow(index.row())
        self.hourModel.submitAll()
        QSqlDatabase.database().commit()
        try:
            client = erppeek.Client(self.erppeekString, self.sqlDbName , self.erppeekUser , self.erppeekUserPassword)
            #client = erppeek.Client.from_config('atlas')
            proxy = client.model('project.task.work')
            if self.debug:
                logger.debug("------------------------------------------------------------")
                logger.debug("id is")
                logger.debug(id)                
            proxy.unlink(int(id))
        except Exception as e:
            QMessageBox.warning(self, "Daily time sheet-Database Error",
                        "Error saving data to server, %s" %(e))
        self.chartDrawn = False
        self.monthGraphDataForMatplot()
        self.getWorkedHours()
        self.btDeleteTaskWork.setEnabled(False)
        self.buttonEdit.setEnabled(False)


    def closeEvent(self, event):
        if self.debug:
            logger.debug('Closing')
            #self.uname = unicode(self.userLabel.text())
        if self.initialized:     
            self.writeSettings()
        event.accept()
        #app.quit()newDbConnection

    def writeSettings(self):
        if self.debug:
                logger.debug('Saving settings')
        file = "settings.ini"
        settings = QSettings(file, QSettings.IniFormat)
      #  settings.setValue("LastUser", QVariant(self.loginDialog.userComboBox.currentText()))
        settings.setValue("MainWindow/Size", QVariant(self.size()))
        settings.setValue("GraphWidth", QVariant(self.dayChartLabel.size().width()))
        settings.setValue("GraphHeight", QVariant(self.dayChartLabel.size().height()))
        settings.setValue("MainWindow/Position",
            QVariant(self.pos()))



    def _setMonthFilter(self, date, updated=0):
        if self.debug:
            logger.debug('_setMonthFilter')
            logger.debug(updated)
        pydate = date.toPyDate()
        dt_D = str(pydate.day)
        dt_M = str(pydate.month)
        dt_Y = str(pydate.year)
        dt = "%s-%s-%s"%(dt_Y, dt_M, dt_D)
        if self.debug:
            logger.debug("date = '%s' and user_id = %s" % (dt, self.currentUserID))

        self.hourModel.setFilter("date = '%s' and user_id = %s" % (dt, self.currentUserID))
        if self.monthChanged:
            #print ("First run 2a")
            self.monthGraphDataForMatplot()
        if self.firstRun:
            if self.debug:
                print ("First run")
            self.monthGraphDataForMatplot()
            self.firstRun = False
        if updated == 1:
            if self.debug:
                print ("Updating")
            #print ("First run 2c")
            self.chartDrawn = False
            self.monthGraphDataForMatplot()
            #self.monthGraphDataForMatplot()

    def monthGraphDataForMatplot(self):
        if self.debug:
            logger.debug('monthGraphDataForMatplot')
        print ("in monthGraphDataForMatplot ")
        import calendar
        #print "cursor on monthGraphDataForMatplot"
        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
        self.lblMessage.setText("Updating chart")
        self.progressMessage.setValue(0)
        self.progressMessage.setVisible(True)
        if (not self.chartDrawn) or self.monthChanged:
            if self.debug:
                logger.debug("Drawing chart")
            self.dayChartLabel.canvas.axis.clear()
            self.dayChartLabel.canvas.figure.clf()
            self.dayChartLabel.canvas.axis = self.dayChartLabel.canvas.figure.add_subplot(111)
            self.dayChartLabel.canvas.figure.subplots_adjust(left=0.03, bottom=0.05, right=0.99, top=0.98)
            self.dayChartLabel.canvas.axis.grid(True)
            self.dayChartLabel.canvas.axis.axis([0, 31, 0, 15])
            majorFormatter = NullFormatter()
            minorLocatorX = MultipleLocator(1)
            minorLocatorY = MultipleLocator(1)
            minorFormattor = FormatStrFormatter('%d')
            self.dayChartLabel.canvas.axis.yaxis.set_minor_locator(minorLocatorY)
            self.dayChartLabel.canvas.axis.xaxis.set_minor_locator(minorLocatorX)
            self.dayChartLabel.canvas.axis.yaxis.set_minor_formatter(minorFormattor)
            self.dayChartLabel.canvas.axis.xaxis.set_major_formatter(majorFormatter)
            self.dayChartLabel.canvas.axis.yaxis.set_major_formatter(majorFormatter)
            self.dayChartLabel.canvas.axis.xaxis.set_minor_formatter(minorFormattor)
            self.dayChartLabel.canvas.axis.xaxis.grid(True, which='minor')
            #self.dayChartLabel.canvas.axis.axis([1, 31, 0, 15])
            date = self.calenderDay.selectedDate()
        #for checking
            month = date.month()
            year = date.year()
            dates = []
            dates1 = []
            hoursWorked = []
            hoursEntered = []
            val = 0
            maxdt = calendar.mdays[month]
    #        print maxdt for t in range
            maxdt = maxdt + 1
            query = QSqlQuery()
            query1 = QSqlQuery()
            for t in range(1, maxdt):
                dates.append(t - .75)
                dates1.append(t - 0.25)
                date_s = "%s-%s-%s"%(year, month, t)
               # if self.debug:
                #    logger.debug("SELECT worked_hours FROM hr_timecard WHERE name = %s AND user_id = %s " % (date_s, self.currentUserID))
              #  query  = 
                query.exec_("SELECT worked_hours FROM hr_timecard WHERE name = '%s' AND user_id = %s " % (date_s, self.currentUserID))
                query1.exec_("SELECT sum(worked_hours)+sum(overtime_hours) as hours FROM smeclient_work WHERE date = '%s' AND user_id = %s " % (date_s, self.currentUserID))
                #print(str(query1.lastQuery()))
                    #print "Query successful  %s" %t
                if query.isActive():
                    query.first()
                    if query.first():
                        date = QDate(year, month, t)
                        self.reformatCalendarPage(date)
                        hoursWorked.append(query.value(0))
                        #hoursEntered.append( query.value(0).toDouble()[0] )
                        #data.append(10)
                        #print "Printing from if with value %s " %data
                    else:
                        if t == (maxdt - 1):
                            hoursWorked.append(.01)
                            #hoursEntered.append(.01)
                        else:
                            hoursWorked.append(val)
                            #hoursEntered.append(val)
                if query1.isActive():
                    query1.first()
                    if query1.first():
                        date = QDate(year, month, t)
                        hoursEntered.append(query1.value(0))
                        #print("There is data in it")
                        #print(query1.value(0))
                    else:
                        if t == (maxdt - 1):
                            #hoursWorked.append(.01)
                            hoursEntered.append(.01)
                        else:
                            hoursEntered.append(val)
            width = 0.50 # the width of the bars
            ###disable for testing
            width = 0.50 # the width of the bars
            #print("Hours worked is")
            #print(hoursWorked)
            #print("Hours entered is")
            #print(hoursEntered)
            p1 = self.dayChartLabel.canvas.axis.bar(dates, hoursWorked, width=width, color='y', align='center')##pg
            p2 = self.dayChartLabel.canvas.axis.bar(dates1, hoursEntered, width, color='r', align='center')##pg
            self.dayChartLabel.canvas.axis.legend((p1[0], p2[0]), ('Worked Hours', 'Data Entered'))##pg
            #self.dayChartLabel.canvas.axis.plot(x, y, 'o')
            ###disable for testing
            self.dayChartLabel.canvas.draw()
        self.chartDrawn = True
        print ("Completed monthGraphDataForMatplot")
        self.progressMessage.setValue(100)
        self.lblMessage.setText(" Chart updated")
        self.progressTimer.start(2000)
        #self.progressMessage.setValue(100)
        #print "cursor off  monthGraphDataForMatplot"
        QApplication.restoreOverrideCursor()
        #if not self.leTimeCardId.text() == "":
            #if self.debug:
                #logger.debug("Saving image to database")
            #pixmap	= QPixmap (QSize(self.dayChartLabel.width()-5,self.dayChartLabel.height()-5))    
            #self.dayChartLabel.render(pixmap) 
            #image_name = "%s.png"%(self.currentUserID)
            #if os.path.exists(image_name):
                #os.remove(image_name)
                #if self.debug:
                    #logger.debug("File exists and removed")
            #else:
                 #if self.debug:
                    #logger.debug("File do not exists")
            #pixmap.save(image_name,  'png')
            #img = open(image_name,  "r")
            #img_data = img.read()
            #id = self.leTimeCardId.text()
            #img_data_encoded = base64.b64encode(img_data)
            #self.saveImageToDatabase(img_data_encoded, id )



    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Escape:
            self.close()
        if event.key() == Qt.Key_F4 and (event.modifiers() &
            Qt.AltModifier):
            self.close()

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.dragPosition = event.globalPos() - self.frameGeometry().topLeft()
            event.accept()

    def mouseMoveEvent(self, event):
        if event.buttons() == Qt.LeftButton:
            try:
                self.move(event.globalPos() - self.dragPosition)
                event.accept()
            except Exception as e:
                if self.debug:
                    logger.debug("Could not move, %s" %(e))
                pass



class JobDelegate(QSqlRelationalDelegate):
    def __init__(self, parent=None, department = None):
        super(JobDelegate, self).__init__(parent)
        self.jobNo = ""

    def setJobNo(self, jobNo):
        #print "Setting job no to %s"%jobNo
        self.jobNo = jobNo
        
        
    def paint(self, painter, option, index):
        myoption = QStyleOptionViewItem(option)
        if index.column() == DATE:
            myoption.displayAlignment |= Qt.AlignRight | Qt.AlignVCenter
        if index.column() == JOBNO:
            myoption.displayAlignment |= Qt.AlignCenter | Qt.AlignVCenter
        if index.column() == HOUR:
            myoption.displayAlignment |= Qt.AlignRight | Qt.AlignVCenter
        if index.column() == UHRS:
            myoption.displayAlignment |= Qt.AlignRight | Qt.AlignVCenter
        if index.column() == OVERTIME:
            myoption.displayAlignment |= Qt.AlignRight | Qt.AlignVCenter
        QSqlRelationalDelegate.paint(self, painter, myoption, index)


    def createEditor(self, parent, option, index):
        if index.column() == JOBNO:
            editor = QLineEdit(parent)
            #editor = QComboBox(parent)
            query = QSqlQuery()
            punctuationRe = QRegExp(r"^[Q0-9][0-9][0-9][0-9][0-9]?[ab]?")
            #queryString = "select `job_No` FROM `temp_jobs` ORDER BY `job_No` DESC"
            queryString = "SELECT DISTINCT(job_no) FROM project_project WHERE job_no IS NOT NULL ORDER BY job_no DESC"
            query.exec_(queryString)
            jobWordList = []
            while query.next():
                jobWordList.append(str(query.value(0)))
            jobCompleter = QCompleter(jobWordList, self)
            completionMode = QCompleter.PopupCompletion
            jobCompleter.setCompletionMode(completionMode)
            editor.setCompleter(jobCompleter)
            editor.setValidator(QRegExpValidator(punctuationRe, self))
            return editor

        if index.column() == TASK:
            editor = QLineEdit(parent)
            taskWordList = []    
            if not self.jobNo  == "":
                query = QSqlQuery()
                query.exec_("SELECT name FROM project_task WHERE job_no = '%s' " %self.jobNo )
                query.first()
                if query.isValid():
                    query.previous()
                    while query.next():
                        taskWordList.append(str(query.value(0)))
                else:
                    task1 = "%s-NC Machining"%self.jobNo              
                    task2 = "%s-Bench Work"%self.jobNo 
                    task3 = "%s-Assembly"%self.jobNo 
                    task4 = "%s-Design"%self.jobNo 
                    taskWordList.append(task1)
                    taskWordList.append(task2)
                    taskWordList.append(task3)
                    taskWordList.append(task4)
            else:
                taskWordList.append("Select task")
            jobCompleter = QCompleter(taskWordList, self)
            completionMode = QCompleter.PopupCompletion
            jobCompleter.setCompletionMode(completionMode)
            jobCompleter.setCaseSensitivity(Qt.CaseInsensitive)
            editor.setCompleter(jobCompleter)
            return editor

        if index.column() == WSUMMARY:
            editor = QLineEdit(parent)
            wSummaryWordList = []    
            query = QSqlQuery()
            #print 'Adding work summary'
            query.exec_("SELECT DISTINCT(work_summary) FROM smeclient_work")
            query.first()
            if query.isValid():
                query.previous()
                while query.next():
                    wSummaryWordList.append(str(query.value(0)))
            jobCompleter = QCompleter(wSummaryWordList, self)
            completionMode = QCompleter.PopupCompletion
            jobCompleter.setCompletionMode(completionMode)
            jobCompleter.setCaseSensitivity(Qt.CaseInsensitive)
            editor.setCompleter(jobCompleter)
            return editor


        if index.column() == HOUR:
            editor = QLineEdit(parent)
            #jobWordList = []
            jobWordList = ["1.0" , "1.25" , "1.5" , "1.75" , "2.0" , "2.25" , "2.5" , "2.75" , "3.0" , "3.25" , "3.5" , "3.75" , "4.0" , "4.25" , "4.5" , "4.75" , "5.0" , "5.25" , "5.5" , "5.75" , "6.0" , "6.25" , "6.5" , "6.75" , "7.0" , "7.25" , "7.5" , "7.75" , "8.0"]
            jobCompleter = QCompleter(jobWordList, self)
            completionMode = QCompleter.InlineCompletion
            jobCompleter.setCompletionMode(completionMode)
            editor.setCompleter(jobCompleter)
            punctuationRe = QRegExp(r"[0-8]?(\.[0,2,5,7][0,5]?)?")
            editor.setValidator(QRegExpValidator(punctuationRe, self))
            return editor

        if index.column() == OVERTIME:
            editor = QLineEdit(parent)
            #jobWordList = QStringList()
            jobWordList = ["1.0" , "1.25" , "1.5" , "1.75" , "2.0" , "2.25" , "2.5" , "2.75" , "3.0" , "3.25" , "3.5" , "3.75" , "4.0" , "4.25" , "4.5" , "4.75" , "5.0" , "5.25" , "5.5" , "5.75" , "6.0" , "6.25" , "6.5" , "6.75" , "7.0" , "7.25" , "7.5" , "7.75" , "8.0"]
            jobCompleter = QCompleter(jobWordList, self)
            completionMode = QCompleter.InlineCompletion
            jobCompleter.setCompletionMode(completionMode)
            editor.setCompleter(jobCompleter)
            punctuationRe = QRegExp(r"[0-8]?(\.[0,2,5,7][0,5]?)?")
            editor.setValidator(QRegExpValidator(punctuationRe, self))
            return editor
        if index.column() == UHRS:
            editor = QLineEdit(parent)
            #jobWordList = QStringList()
            jobWordList  = ["1.0" , "1.25" , "1.5" , "1.75" , "2.0" , "2.25" , "2.5" , "2.75" , "3.0" , "3.25" , "3.5" , "3.75" , "4.0" , "4.25" , "4.5" , "4.75" , "5.0" , "5.25" , "5.5" , "5.75" , "6.0" , "6.25" , "6.5" , "6.75" , "7.0" , "7.25" , "7.5" , "7.75" , "8.0"]
            jobCompleter = QCompleter(jobWordList, self)
            completionMode = QCompleter.InlineCompletion
            jobCompleter.setCompletionMode(completionMode)
            editor.setCompleter(jobCompleter)
            punctuationRe = QRegExp(r"[0-8]?(\.[0,2,5,7][0,5]?)?")
            editor.setValidator(QRegExpValidator(punctuationRe, self))
            return editor
        else:
            return QSqlRelationalDelegate.createEditor(self, parent, option, index)

    def setEditorData(self, editor, index):
        if index.column() == TASK:
            prevJobNo = "Select task"
            editor.setText(prevJobNo)

        #if index.column() == PROCESSID:
            ##print "Department Id is %s" %departmentId
            #if self.departmentId == "1":
                #text = "Bench"
            #elif self.departmentId == "2":
                #text = "NC Milling"
            #elif self.departmentId == "3":
                #text = "EDM Wirecutting"
            #elif self.departmentId == "4":
                #text = "Design"
            #elif self.departmentId == "5":
                #text = "Follow Up"
            ##print text
            ##text = index.model().data(index, Qt.DisplayRole).toString()
            ##text = "Bench"
            #editor.setCurrentIndex(editor.findText(text))
            ##editor.setText(text)
        else:
            QSqlRelationalDelegate.setEditorData(self, editor, index)


    def setModelData(self, editor, model, index):
#        print "adjusting data"
#        if index.column() == PROCESSID:
#            data = 1
#            model.setData(index, QVariant(data))
#        else:
        QSqlRelationalDelegate.setModelData(self, editor, model, index)

#################################################################

