###################################################################################
# -*- coding: utf-8 -*-                                                           #
#!/usr/bin/env python                                                             #
# SMETime Data Manager for SME's                                                  #
# Copyright (C) 2008 Anil anilet@gmail.com                                        #
#                                                                                 #
# This program is free software; you can redistribute it and/or                   #
# modify it under the terms of the GNU General Public License                     #
# version 2 as published by the Free Software Foundation.                         #
#                                                                                 #
# This program is distributed in the hope that it will be useful,                 #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                  #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                   #
# GNU General Public License for more details.                                    #
#                                                                                 #
# You should have received a copy of the GNU General Public License               #
# along with this program; if not, write to the Free Software                     #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. #
###################################################################################
import logging
logger = logging.getLogger('Login dialog')
FORMAT = '[%(levelname)-7s] [%(name)-35s] - %(message)s'
logging.basicConfig(level=logging.DEBUG, format=FORMAT)
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtSql import *
from PyQt5.QtWidgets import QDialog
from PyQt5.QtWidgets import QApplication
import base64
from ui.ui_login import Ui_LoginDialog # Login dialog

class LoginDialog (QDialog, Ui_LoginDialog):
    closeAppSignal = pyqtSignal()
    def __init__(self, parent=None,  db = None):
        super(LoginDialog, self).__init__(parent)
        self.setupUi(self)
        self.userName = None
        self.valid = True
        self.debug = True
        # when dialog closed delete it
        self.setAttribute(Qt.WA_DeleteOnClose)
        # Remove the edges of the window
        self.setWindowFlags(self.windowFlags() | Qt.MSWindowsFixedSizeDialogHint )
        self.setWindowTitle( "SMETime Odoo Client- Please Login")
        self.defaultUser = ""
        self.cbPartnerId.setVisible(False)
        self.cbId.setVisible(False)
        #qsname = "settings.ini"
        #self.mainSettings = QSettings(qsname, QSettings.IniFormat)
        self.fillUserCombo()
        self.loginButton.setText("Login")
       # self.btn1.clicked.connect(self.buttonClicked)
        self.exitButton.clicked.connect(self.closeApp)
        self.exitButtonError.clicked.connect(self.closeApp)
        self.loginButton.clicked.connect(self.doLogin)
        self.closeAppSignal.connect(self.closeApp)
        self.stackedWidget.setCurrentIndex(0)
        self.getLogo()
        self.show()
        #self.show()
        
        
    def closeApp(self):	
        if self.debug:
            logger.debug('closeApp') 
        QApplication.quit() # Close application


    def fillUserCombo(self):
        if self.debug:
            logger.debug('fillUserCombo from login') 
        query = QSqlQuery()
        query.exec_("select login,id,partner_id from res_users where active=True order by login ASC")
        while query.next():
            name = str(query.value(0))
            self.cbUser.addItem(name)
            self.cbId.addItem(str(query.value(1)))
            self.cbPartnerId.addItem(str(query.value(2)))
        settings = QSettings()
        uname = settings.value("LastUser")
        #remember = settings.value("RememberPass")
        #self.loadDefaultUser = settings.value("loadDefaultUser").toString()
        #self.defaultUser = settings.value("LastUser").toString()
        self.cbUser.setCurrentIndex(self.cbUser.findText(uname))        
        if not query.isActive():
            self.lblErrorMessage.setText( "<html><head/><body><p><span style=\" color:#c00000;\">Failed to open database.<br> Please check your login details, <br>Ask your administrator to reset  password</span></p></body></html>")
            self.valid = False
    
    def doLogin(self):
        if self.debug:
            logger.debug('doLogin from login') 
        """	Method that performs the login
        """
        if  self.valid:
            self.userName = self.cbUser.currentText()
            settings = QSettings()
            settings.setValue("LastUser", QVariant(self.userName))
           # self.mainSettings.setValue("LastUser", QVariant(self.userName))
            return self.userName, True
        else:
            return self.userName, False

    
    def getLogo(self):
        """
            Get company logo from Odoo and save to root folder
        """
        if self.debug:
            logger.debug('getLogo ')
        query = QSqlQuery()
        ba1 = QByteArray()
        pic = QPixmap()
        query.exec_("SELECT decode(replace(encode(logo_web,'escape')::text,E'\012',''),'base64') as logo from res_company")
        while query.next():
            ba1 = query.value(0)
        if ba1:
            if self.debug:
                logger.debug('Loading static image ')
            fileName = "logo.png"

        pic.loadFromData( ba1)
        pic.save(fileName)
       # pic = pic.scaled(QSize(28,28),Qt.KeepAspectRatio,Qt.SmoothTransformation)
        self.loginPixmap.setPixmap(pic)
        #self.btUser.setText(self.currentUserDisplayName )
        
        
            
        
#    def check_password(enc_password):
#        """
#        Returns a boolean of whether the raw_password was correct. Handles
#        encryption formats behind the scenes.
#        """
#        password = base64.b64decode(enc_password)
#        #algo, salt, hsh = enc_password.split('$')
#        print "Password is %s"%password
#        return password
