-- Table: smeclient_work

DROP TABLE smeclient_work;

CREATE TABLE smeclient_work
(
  id serial NOT NULL,
  name character varying(8) NOT NULL, -- Job No
  task character varying(100) NOT NULL, -- Task
  work_summary text NOT NULL, -- Work Summary
  worked_hours double precision, -- Worked Hours
  overtime_hours double precision, -- Overtime Hours
  unattended_hours double precision, -- Unatteded Hours
  date date, -- Worked Date
  user_id integer NOT NULL, -- Done by
  create_date timestamp without time zone, -- Created on
  create_uid integer, -- Created by
  write_date timestamp without time zone, -- Last Updated on
  write_uid integer, -- Last Updated by
  workid integer, -- Work Id
  CONSTRAINT smeclient_work_pkey PRIMARY KEY (id),
  CONSTRAINT smeclient_work_create_uid_fkey FOREIGN KEY (create_uid)
      REFERENCES res_users (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL,
  CONSTRAINT smeclient_work_user_id_fkey FOREIGN KEY (user_id)
      REFERENCES res_users (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL,
  CONSTRAINT smeclient_work_write_uid_fkey FOREIGN KEY (write_uid)
      REFERENCES res_users (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL
)
WITH (
  OIDS=FALSE
);
ALTER TABLE smeclient_work
  OWNER TO odoo;
COMMENT ON TABLE smeclient_work
  IS 'For smeclient';
COMMENT ON COLUMN smeclient_work.name IS 'Job No';
COMMENT ON COLUMN smeclient_work.task IS 'Task';
COMMENT ON COLUMN smeclient_work.work_summary IS 'Work Summary';
COMMENT ON COLUMN smeclient_work.worked_hours IS 'Worked Hours';
COMMENT ON COLUMN smeclient_work.overtime_hours IS 'Overtime Hours';
COMMENT ON COLUMN smeclient_work.unattended_hours IS 'Unatteded Hours';
COMMENT ON COLUMN smeclient_work.date IS 'Worked Date';
COMMENT ON COLUMN smeclient_work.user_id IS 'Done by';
COMMENT ON COLUMN smeclient_work.create_date IS 'Created on';
COMMENT ON COLUMN smeclient_work.create_uid IS 'Created by';
COMMENT ON COLUMN smeclient_work.write_date IS 'Last Updated on';
COMMENT ON COLUMN smeclient_work.write_uid IS 'Last Updated by';
COMMENT ON COLUMN smeclient_work.workid IS 'Work Id';


-- Index: smeclient_work_date_index

-- DROP INDEX smeclient_work_date_index;

CREATE INDEX smeclient_work_date_index
  ON smeclient_work
  USING btree
  (date);

-- Index: smeclient_work_name_index

-- DROP INDEX smeclient_work_name_index;

CREATE INDEX smeclient_work_name_index
  ON smeclient_work
  USING btree
  (name COLLATE pg_catalog."default");

-- Index: smeclient_work_user_id_index

-- DROP INDEX smeclient_work_user_id_index;

CREATE INDEX smeclient_work_user_id_index
  ON smeclient_work
  USING btree
  (user_id);