###################################################################################
# -*- coding: utf-8 -*-                                                           #
#!/usr/bin/env python                                                             #
# SMETime Data Manager for SME's                                                  #
# Copyright (C) 2008 Anil anilet@gmail.com                                        #
#                                                                                 #
# This program is free software; you can redistribute it and/or                   #
# modify it under the terms of the GNU General Public License                     #
# version 2 as published by the Free Software Foundation.                         #
#                                                                                 #
# This program is distributed in the hope that it will be useful,                 #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                  #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                   #
# GNU General Public License for more details.                                    #
#                                                                                 #
# You should have received a copy of the GNU General Public License               #
# along with this program; if not, write to the Free Software                     #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. #
###################################################################################



from PyQt5.QtWidgets import  QWidget
from PyQt5 import QtCore, QtGui, QtWidgets, QtSql
import datetime
import time
import logging
FORMAT = '[%(levelname)-7s] [%(name)-35s] - %(message)s'
logging.basicConfig(level=logging.DEBUG, format=FORMAT)
logger = logging.getLogger('calendar')
DATE_FORMAT = "MMM d, yyyy"
ID, FNAME, LNAME, POSITION = range(4)
WORKID = 0
JOBNO = 1
PARTID = 2
PROCESSID = 3
DATE = 12
WDAY = 4
WMONTH = 5
WYEAR = 6
EMPLID = 7
HOUR = 8
OVERTIME = 9
UHRS = 10
NOTES = 11
ACQUIRED = 1
WIDH = 20
WIDH1 = 80
EMPLOYEEID = ""

class CalendarWidgetInternal(QWidget):
    selectedDateChanged = QtCore.pyqtSignal()
    monthChanged = QtCore.pyqtSignal()
    
    def __init__(self, parent=None,  currentUser = None):
        super(CalendarWidgetInternal, self).__init__(parent)
        self.m_selectedDate = QtCore.QDate()
        self.m_selectedDay =  0
        self.debug = True
        self.m_currentDate = QtCore.QDate()
        self.m_firstDayOfWeek = QtCore.Qt.Monday
        self.m_monthStartAt = self.m_monthEndAt = 0
        self.m_headHeight = self.m_cellHeight = self.m_cellWidth = 30
        self.m_sideHeadWidth = 50
        self.currentUserId = currentUser
        self.m_selectedUser = currentUser
        self.month = None
        self.year = None
        self.currentMonth = None
        self.dirty = True
        self.setSizePolicy(QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding,
                                      QtWidgets.QSizePolicy.Fixed))
        now = datetime.datetime.now()
        self.m_currentDate = now
        


    def setSelectedDate(self, date):
        if (date.isValid()):
            if (date != self.m_selectedDate):
                self.m_selectedDate = date
                self.update()
                #self.s
                self.selectedDateChanged.emit()
                #self.emit(SIGNAL("selectedDateChanged(date)"),date)
    
    def setSelectedMonth(self, month, year):
        if self.debug:
            logger.debug('setSelectedMonth')
            logger.debug(month)
        self.month = month
        self.year = year
        self.update()
        self.monthChanged.emit()
        #self.emit(SIGNAL("monthChanged(month)"),month)

    def redraw(self):
        print (" Redrawing ")
       # self.clear()
        self.repaint()
        
    def selectedDate(self):
        return self.m_selectedDate
        
    def selectedMonth(self):
        return self.month

    def selectedDay(self):
        return self.m_selectedDay
        
    def selectedUser(self):
        return self.m_selectedUser
      
    #def selectedDateChanged(self):
        #self.currentDateEdit.setDate(self.calendar.selectedDate())


#
#
    def setFirstDayOfWeek(self, dayOfWeek):
        self.m_firstDayOfWeek = dayOfWeek
       # self.computeMonthBoundaries()
        self.update()
        
    def rowCount(self):    
        query = QtSql.QSqlQuery()
        query.exec_("SELECT COUNT('login') FROM  res_users WHERE  active=True")
        while query.next():
            return query.value(0).toInt()

    def employeeList(self):
        query = QtSql.QSqlQuery()
        query.exec_("select login from res_users where active=True order by login ASC")
        employees = []
        while query.next():
            employees.append(str(query.value(0)))
        #print employees
        return employees

    def firstDayOfWeek(self):
        return self.m_firstDayOfWeek

    #def event(self,event)
	#if (event->type() == QEvent::ToolTip):
	  #QHelpEvent *helpEvent = static_cast<QHelpEvent *>(event)

		#if (isDate(helpEvent->pos())) {
			#if (m_delegate) {
				#QString tip = m_delegate->toolTipText(findDate(helpEvent->pos()));
				#QToolTip::showText(helpEvent->globalPos(), tip);
				#return true;
			#}
		#}

		#QToolTip::hideText();
	#}

	#return QWidget::event(event);
#}

    def paintEvent(self, event=None):
        QtWidgets.QApplication.setOverrideCursor(QtGui.QCursor(QtCore.Qt.WaitCursor))
        if not self.month == self.currentMonth:
            if self.debug:
               logger.debug("Painting")
               logger.debug('Month changed-New month is %s' %self.month) 
               logger.debug('Year changed-New year is %s' %self.year) 
            #print ("Month changed")
        self.setGeometry(10,0,self.parentWidget().width(),self.parentWidget().height()-0)
        painter = QtGui.QPainter(self)
        pen = QtGui.QPen()
        pen.setStyle(QtCore.Qt.SolidLine)
        pen.setColor(QtCore.Qt.black)
        painter.setPen(pen)
        #painter.setPen(self.palette().color(QPalette.Active, QPalette.Dark))
        employees = self.employeeList()
        option = QtGui.QTextOption(QtCore.Qt.AlignCenter)
        option.setWrapMode(QtGui.QTextOption.WordWrap)   
        m_headHeight = 24
        raws =  len(employees)
        if self.debug:
               logger.debug("Raws are %s" %raws)
        self.m_cellHeight = (self.height() - 60) / len(employees)
        #print ("Cell height is %s" %self.m_cellHeight)
        self.m_cellWidth = (self.width() - self.m_sideHeadWidth  - 20 )/ 32
        painter.drawLine(60, 0, 60, self.height() - 20);
        rect = QtCore.QRect
        rectDays = QtCore.QRect
        query = QtSql.QSqlQuery()
        for y in range(raws):
            rect =QtCore.QRect( 1, 25 + self.m_cellHeight * y , 54,  self.m_headHeight)
            self.drawHeaderCell(painter, rect, employees[y])
        #rect =QRect( (50 + m_cellWidth/4 + m_cellWidth * y), self.height() - 50 , m_cellWidth,  15)
        painter.drawLine(30, 20 + self.m_cellHeight * (y +1), self.width(), 20 + self.m_cellHeight * (y+1));
        for y in range(1, 32):
            #rect = QRect((m_cellWidth * y), 0, y != 6 ? m_cellWidth : width() - (m_cellWidth * y) - 1, m_headHeight)
            rect =QtCore.QRect( (30 + self.m_cellWidth * y), self.height() - 45 , self.m_cellWidth,  15)
            for x in range(raws):
                painter.setBrush(QtGui.QColor(213, 115, 159))
                #while y < 
                date = "%s-%s-%s" %(self.year, self.month,y)
                query.exec_("SELECT worked_hours FROM hr_timecard WHERE name = '%s' AND user_id = %s " % (date, self.getUserID(employees[x])))
               # print (query.lastQuery())
                rectDays1 =  QtCore.QRect( )
                text=""
                printText = ""
                painter.setBrush(QtGui.QColor(213, 115, 159))
                
                while query.next():
                    #print ("Query is valid")
                    hours = ""
                    hours = str(query.value(0))
                   # print hours
                    painter.setBrush(QtCore.Qt.green)
                    rectDays1 =  QtCore.QRect( (60 + self.m_cellWidth * (y - 1)), 20 + self.m_cellHeight * (x ) , self.m_cellWidth,  self.m_cellHeight)
                    text = self.hoursWorked(date,self.getUserID(employees[x]))
                    printText = text + "/" + hours
                    #print (printText)
                    
                rectDays =  QtCore.QRect( (60 + self.m_cellWidth * (y - 1)), 20 + self.m_cellHeight * (x ) , self.m_cellWidth,  self.m_cellHeight)
                #text = self.hoursWorked(y,self.getUserID(employees[x]))
               # painter.drawText(QRectF( (60 + self.m_cellWidth * (y - 1)), 20 + self.m_cellHeight * (x ) , self.m_cellWidth,  self.m_cellHeight), text, option)
                self.paintDaysRect(painter,  rectDays)
                self.drawHours(painter, rectDays1, printText)
                #self.drawHours(painter, rectDays, text)
            if not y == 0:
                 self.drawDays(painter, rect, y)
            self.currentMonth = self.month
            self.dirty = False
            QtWidgets.QApplication.restoreOverrideCursor()


	
    def getUserID(self,  user):
        query = QtSql.QSqlQuery()
        query.exec_("SELECT id FROM res_users where active=True AND login = '%s'" % user)
        while query.next():
            ID = ""
            ID = str(query.value(0))
            return ID
            #print ID
            if self.debug:
                logger.debug('user ID is %s' % ID)
            return ID


    def drawHeaderCell(self, painter, rect,  dayName):
        painter.save()
        painter.drawText(rect, QtCore.Qt.AlignRight | QtCore.Qt.TextSingleLine, str(dayName))
        painter.restore()
    
    def drawDays(self, painter, rect,  day):
        painter.save()
        painter.drawText(rect, QtCore.Qt.AlignCenter | QtCore.Qt.TextSingleLine, str(day))
        painter.restore()
        
    def drawHours(self, painter, rect,  day):
        painter.save()
        painter.drawText(rect, QtCore.Qt.AlignCenter | QtCore.Qt.TextSingleLine, str(day))
        painter.restore()
    
    
    def paintDaysRect(self, painter, rect):
        painter.save()
        painter.drawRect(rect)
        #painter.drawText(rect, Qt.AlignCenter | Qt.TextSingleLine, QString.number(day))
        painter.restore()
        
    def hoursWorked(self, day,user):
        query1 = QtSql.QSqlQuery()
        if self.debug:
               logger.debug("Printing from hoursworked %s, %s" %(day, user))
        query1.exec_("SELECT sum(worked_hours)+sum(overtime_hours) as hours FROM smeclient_work WHERE date = '%s' AND user_id = %s " % (day, user))
        if self.debug:
               logger.debug('Query is %s' % query1.lastQuery())
        #print (query.lastQuery())
        while query1.next():
            hours = ""
            hours = str(query1.value(0))
            return hours
            if self.debug:
                logger.debug('user ID is %s' % hours)
            return hours

#
    def drawEmptyCell(self, painter, rect):
        painter.drawRect(rect)
#

    def drawItemCell(self,  painter, rect, weekday, day):
        painter.save()

        if (day == m_selectedDate.day()):
            painter.setPen(palette().color(QtGui.QPalette.Active, QtGui.QPalette.Dark))
            #painter.setBrush(palette().brush(hasFocus() ? QPalette:Active : QPalette:Inactive, QPalette.Highlight))
            painter.drawRect(rect)
           # painter.setPen(palette().color(hasFocus() ? QPalette.Active : QPalette.Inactive, QPalette.HighlightedText))
            painter.drawText((rect.topLeft() + QtGui.QPoint(4, 12)), ("%s" % day))
        else:
            painter.setPen(palette().color(QtGui.QPalette.Active, QtGui.QPalette.Dark))
            painter.setBrush(palette().brush(QtGui.QPalette.Base))
            painter.drawRect(rect)
           # painter.setPen(((weekday == QtCore.Qt.Saturday) || (weekday == QtCore.Qt.Sunday)) ? QColor(Qt.red) : palette().color(QPalette.Active, QPalette.Text))
            painter.drawText((rect.topLeft() + QtGui.QPoint(4, 12)), ("%s" % day))



    def findDate(self, pos):
        #print " In findDate"
        employees = self.employeeList()
        print("Pos x = %s y = %s cell width is %s ht is %s" %(pos.x(),  pos.y(),  self.m_cellWidth, self.m_cellHeight ))
        if  not (self.isDate(pos)): 
            return QDate()
        
        columnID = (pos.x() -28) / self.m_cellWidth
        rowID = (pos.y() - 20) / self.m_cellHeight

        #offset = ((rowID * (len(employees))) + columnID) + 1
       # print (" In findDate -columnID is %s rowID is %s employee is %s"%(columnID, rowID, employees[rowID] ))
        print (" In findDate -columnID is %s rowID is %s "%(columnID, rowID))
        self.m_selectedUser = "%s"%employees[rowID] 
        #self.setDateForWorkView(QDate(2012,7, columnID),  id)
        #day = offset - self.m_monthStartAt + 1
       # return QDate(self.m_selectedDate.year(),self.m_selectedDate.month(), offset)
        self.m_selectedDay = columnID
        return QDate(2012,7, columnID)
        
#
    def isDate(self, pos):
        rect = QtCore.QRect(60, 20+ 1, self.width() - 115, self.height() -61)
        #print " In is date"
        return rect.contains(pos)

    def keyPressEvent(self, event):
        key = event.key()
        if event.key() in (Qt.Key_Left, Qt.Key_Right):
           # offset = ((key == Qt.Key_Left) ? -1 : 1)
            offset = 1
          #modifiers = Qt.KeyboardModifiers
          #modifiers = event.modifiers()
            self.setSelectedDate(m_selectedDate.addDays(offset))
            event.accept()
        
        elif event.key() in (Qt.Key_Up, Qt.Key_Down):
          #offset = ((key == Qt::Key_Up) ? -7 : 7)
          offset = -7
          self.setSelectedDate(m_selectedDate.addDays(offset))
          event.accept()
          
        elif event.key() in (Qt.Key_Return, Qt.Key_Enter):
         # self.emit(SIGNAL("activated()"))
          event.accept()
          
        else:
                QWidget.keyPressEvent(self, event)
            
    
    def mouseDoubleClickEvent(self, event):
        if (self.isDate(event.pos())):
        #    z#print "Clicked inside day"
            #self.emit(SIGNAL("activated()"))
            date = self.findDate(event.pos())
            print ("DBlClicked date is %s"%(date.toString(DATE_FORMAT)))
            self.selectedDateChanged.emit()
            #self.emit(SIGNAL("selectedDateChanged(QDate)"),date)
            


