###################################################################################
# -*- coding: utf-8 -*-                                                           #
#!/usr/bin/env python                                                             #
# SMETime Data Manager for SME's                                                  #
# Copyright (C) 2008 Anil anilet@gmail.com                                        #
#                                                                                 #
# This program is free software; you can redistribute it and/or                   #
# modify it under the terms of the GNU General Public License                     #
# version 2 as published by the Free Software Foundation.                         #
#                                                                                 #
# This program is distributed in the hope that it will be useful,                 #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                  #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                   #
# GNU General Public License for more details.                                    #
#                                                                                 #
# You should have received a copy of the GNU General Public License               #
# along with this program; if not, write to the Free Software                     #
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. #
###################################################################################
import logging
logger = logging.getLogger('Setup dialog')
FORMAT = '[%(levelname)-7s] [%(name)-35s] - %(message)s'
logging.basicConfig(level=logging.DEBUG, format=FORMAT)
from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtSql import *
from PyQt5.QtWidgets import QDialog, QMessageBox
from PyQt5.QtWidgets import QApplication
import erppeek
import base64
import os
from ui.Ui_settings import Ui_SetupDialog # 

class SetupDialog (QDialog, Ui_SetupDialog):
    closeAppSignal = pyqtSignal()
    def __init__(self, parent=None,  db = None):
        super(SetupDialog, self).__init__(parent)
        self.setupUi(self)
        self.debug = True
        self.erppeekString = ""
        self.sqlDbHost = ""
        self.sqlDbName = ""
        self.sqlDbUser = ""
        self.sqlDbPasswd = ""
        self.erppeekUser = ""
        self.erppeekUserPassword = ""
        
        
        
        # when dialog closed delete it
        self.setAttribute(Qt.WA_DeleteOnClose)
        # Remove the edges of the window
        self.setWindowFlags(self.windowFlags() | Qt.MSWindowsFixedSizeDialogHint )
        self.setWindowTitle( "SMETime Odoo Client- Database Setup")
        self.cancelButton.clicked.connect(self.closeApp)
        self.testButton.clicked.connect(self.testConnection)
        self.saveButton.clicked.connect(self.saveSettings)
        self.closeAppSignal.connect(self.closeApp)
        self.getSettings()
        self.show()
        
        
    def closeApp(self):	
        if self.debug:
            logger.debug('closeApp') 
        self.close()
        QApplication.quit() # Close application
        
    def saveSettings(self):
        file = "dbsettings.ini"
        if self.debug:
           logger.debug('Saving settings')
        settings = QSettings(file, QSettings.IniFormat)
        settings.setValue("sqlDbHost", self.ipEdit.text())
        settings.setValue("sqlDbName", self.dbNameEdit.text())
        settings.setValue("sqlDbUser", self.dbUserNameEdit.text())
        encodedPassword = base64.b64encode (bytes(self.dbPasswordEdit.text(), "utf-8"))
        encodedPassword = "{0}".format(encodedPassword.decode('UTF-8'))
        settings.setValue("sqlDbPasswd", str(encodedPassword))
        settings.setValue("erppeekUser", self.odooUserEdit.text())
        encodedErppeekPassword = base64.b64encode (bytes(self.odooUserPassEdit.text(), "utf-8"))
        encodedErppeekPassword = "{0}".format(encodedErppeekPassword.decode('UTF-8'))
        settings.setValue("erppeekUserPassword", str(encodedErppeekPassword))
        settings.setValue("LastUser", "")
        settings.sync()
        #self.closeApp()
            
    def testConnection(self):
        """Create a new connection to the Db
                 using information obtained from the XML file configuration
                """
        QSqlDatabase.removeDatabase("qt_sql_default_connection")
        connection = QSqlDatabase.addDatabase('QPSQL' )
        connection.setHostName(self.ipEdit.text() )
        connection.setDatabaseName(self.dbNameEdit.text())
        connection.setUserName(self.dbUserNameEdit.text())
        connection.setPassword(self.dbPasswordEdit.text())
       # logger.debug('connection string is  %s -  %s -  %s -  %s' %(self.sqlDbHost, self.sqlDbName, self.sqlDbUser, self.decodedPassword))
        if not connection.open():
            QMessageBox.critical(None,
            self.tr("Error opening connection "),
            self.tr("""Error opening connection - %s""" % connection.lastError().text()),
            QMessageBox.StandardButtons(\
                QMessageBox.Ok))
        else:
           #self.erppeekString = "'http://%s:8069'"  %(self.ipEdit.text())
           self.erppeekString = "http://" + self.ipEdit.text() + ":8069"
           print(self.erppeekString)
           if self.checkErpLogin():
               self.saveButton.setEnabled(True)
               self.testButton.setEnabled(False)
               self.successLabel.setVisible(True)
        
    
    
    def checkErpLogin(self):
        try:
            # Login
            if self.debug:
                logger.debug( "Connecting to odoo from setup")
                logger.debug(self.erppeekString)
                logger.debug(self.dbNameEdit.text())
                logger.debug(self.odooUserEdit.text())
                logger.debug(self.odooUserPassEdit.text() )
            #client = erppeek.Client(self.erppeekString)
            client = erppeek.Client(self.erppeekString, self.dbNameEdit.text(), self.odooUserEdit.text(), self.odooUserPassEdit.text())
            #client = erppeek.Client(self.erppeekString, self.dbNameEdit.text(), self.odooUserEdit.text(), self.odooUserPassEdit.text())
            proxy = client.model('res.users')
            users = proxy.browse([])
            #for user in users:
            #    print("{user.id}, {user.name}".format(user=user))
            return True
        except Exception as e:
            self.valid = False
            QMessageBox.critical(None,
            self.tr("Error opening connection "),
            self.tr("""Failed to open database. <br>Please check your login details, <br>Ask your administrator to reset password. <br> Error message is - %s""" %e),
            QMessageBox.StandardButtons(\
                QMessageBox.Ok))
            return False
    
    def getSettings(self):
      if self.debug:
         logger.debug('get settings from set up')
      file = "dbsettings.ini"
      if os.path.isfile(file):
            settings = QSettings(file, QSettings.IniFormat)
            self.sqlDbHost = settings.value("sqlDbHost", "")
            self.ipEdit.setText(self.sqlDbHost)
            self.sqlDbName = settings.value("sqlDbName", "")
            self.dbNameEdit.setText(self.sqlDbName)
            self.sqlDbUser = settings.value("sqlDbUser", "")
            self.dbUserNameEdit.setText(self.sqlDbUser)
            self.sqlDbPasswd = str(base64.b64decode(settings.value("sqlDbPasswd", "")), encoding='UTF-8')
            self.dbPasswordEdit.setText(self.sqlDbPasswd)
            self.erppeekUser = settings.value("erppeekUser", "")
            self.odooUserEdit.setText(self.erppeekUser)
            self.erppeekUserPassword = str(base64.b64decode(settings.value("erppeekUserPassword", "")), encoding='UTF-8')
            self.odooUserPassEdit.setText(self.erppeekUserPassword)
            self.erppeekString = "http://" + self.ipEdit.text() + ":8069"
            print (self.erppeekString)
            
         # if sqlDbHost == "" or sqlDbUser == "" or sqlDbName == "" or decodedPassword == "" or:
         #    logger.debug('Creating new settings file')
      else:
            self.ipEdit.setText("")
            self.dbNameEdit.setText("")
            self.dbUserNameEdit.setText("")
            self.dbPasswordEdit.setText("")
            self.testButton.setEnabled(True)
            self.saveButton.setEnabled(False)
            self.successLabel.setVisible(False)
                  
       
#    def check_password(enc_password):
#        """
#        Returns a boolean of whether the raw_password was correct. Handles
#        encryption formats behind the scenes.
#        """
#        password = base64.b64decode(enc_password)
#        #algo, salt, hsh = enc_password.split('$')
#        print "Password is %s"%password
#        return password
